# 2022-2-13  cp from cikuu/bin/es.py 
import json,fire,sys, redis,os #, spacy
#from elasticsearch import helpers,Elasticsearch
#from .config import config, readline
from so import * 

class ES(object):

	def __init__(self, host='127.0.0.1',port=9200): 
		self.es = Elasticsearch([ f"http://{host}:{port}" ])  

	def hello(self): print (self.es)

	def addfolder(self, folder:str, corpus:str, pattern=".txt", idxname:str='docbase'): 
		''' folder -> docbase, 2022.1.23 '''
		print("addfolder started:", folder, idxname, self.es, flush=True)
		if not self.es.indices.exists(idxname): self.es.indices.create(idxname, config)
		for root, dirs, files in os.walk(folder):
			for file in files: 
				if file.endswith(pattern):
					self.es.index(index=idxname,id = f"{corpus}-{file}",  body = {"filename":file,'corpus':corpus,'body':open(f"{folder}/{file}",'r', encoding='utf-8').read().strip() })
					print (file, folder, flush=True)
		print("addfolder finished:", folder, idxname, self.es, flush=True)

	def loadjukuu(self, infile, idxname, batch=100000): 
		''' 2022.1.11 '''
		print("loadsource started:", infile, idxname, self.es, flush=True)
		if not self.es.indices.exists(idxname): self.es.indices.create(idxname, config)
		actions=[]
		for line in readline(infile):  #{"_id": "140948871-9", "_source": {"rid": "10", "uid": "25110374", "sc": 14, "md5": "da891a7d81f7a5e43b571168cc483b6c dba0b4c99ef37cadfc4bacd61fcefa5b d6b199bfae35246564c598ac78d84c91 38a945eeff5b5a587a26dcc6560e0061 58605af6b50b01f15c0cc3ee2aa75e33 c30566c355ae09ea68673e2940d49d0a 7972c906aef51380310363093e141ef8 dabdd545400415da6d29125bf872"}}
			try:
				arr = json.loads(line)
				arr['content'] = arr['en'] + ' ' + arr['zhseg']
				del arr['zhseg'] 
				actions.append({'_op_type':'index', '_index':idxname, "_source": arr})
				if len(actions) > batch : 
					helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
					actions=[]
					print(arr, flush=True) 
			except Exception as ex:
				print(">>callback ex:", ex, line)
		helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
		print("loadsource finished:", infile,idxname)

	def idsource(self, infile, idxname, batch=100000): 
		''' {"_id": "140948871-9", "_source": {"rid": "10", "uid": "25110374", "sc": 14, "md5": "da891a7d81f7a5e43b571168cc483b6c dba0b4c99ef37cadfc4bacd61fcefa5b d6b199bfae35246564c598ac78d84c91 38a945eeff5b5a587a26dcc6560e0061 58605af6b50b01f15c0cc3ee2aa75e33 c30566c355ae09ea68673e2940d49d0a 7972c906aef51380310363093e141ef8 dabdd545400415da6d29125bf872"}}
		added 2022.2.11 '''
		print("idsource started:", infile, idxname, self.es, flush=True)
		if not self.es.indices.exists(idxname): self.es.indices.create(idxname, config)
		actions=[]
		for line in readline(infile):  
			try:
				arr = json.loads(line)
				if not '_source' in arr : arr = {"_source": arr} # source only njson 
				arr.update({'_op_type':'index', '_index':idxname})
				actions.append(arr)
				if len(actions) > batch : 
					helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
					actions=[]
					print(arr["_id"], flush=True) 
			except Exception as ex:
				print(">>callback ex:", ex, line)
		helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
		print("idsource finished:", infile,idxname)

	def loadsnts(self, infile, idxname, batch=100000): 
		''' {"eid": 63885890, "ver": "20", "rid": 10, "uid": 6641784, "sc": 28, "snts": ["Robinson was born in an honest, middle - class -- added 2021.12.1 '''
		print("loadsnts started:", infile, flush=True)
		if not self.es.indices.exists(idxname): self.es.indices.create(idxname, config)
		actions=[]
		for line in readline(infile):  #{"_id": "140948871-9", "_source": {"rid": "10", "uid": "25110374", "sc": 14, "md5": "da891a7d81f7a5e43b571168cc483b6c dba0b4c99ef37cadfc4bacd61fcefa5b d6b199bfae35246564c598ac78d84c91 38a945eeff5b5a587a26dcc6560e0061 58605af6b50b01f15c0cc3ee2aa75e33 c30566c355ae09ea68673e2940d49d0a 7972c906aef51380310363093e141ef8 dabdd545400415da6d29125bf872"}}
			try:
				arr = json.loads(line)
				arr["md5"] = sntmd5( arr['snts'])
				id = f"{arr['eid']}-{arr['ver']}"
				del arr['snts']
				del arr['eid']
				del arr['ver']
				actions.append({'_op_type':'index', '_index':idxname,  "_id": id, "_source": arr})
				if len(actions) > batch : 
					helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
					actions=[]
					print(line) 
			except Exception as ex:
				print(">>callback ex:", ex, line)
		helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
		print("index_json finished:", infile,idxname)

	def eevjson(self, infile, idxname, batch=100000,  rhost='127.0.0.1', rport=6662, rdb=0): 
		''' 2021.12.8 '''
		r = redis.Redis(rhost, port=rport, db=rdb, decode_responses=True)
		print("load eevjson started:", infile, r, self.es, flush=True)
		if not self.es.indices.exists(idxname): self.es.indices.create(idxname, config)
		actions=[]
		for line in readline(infile):  #{"_id": "140948871-9", "_source": {"rid": "10", "uid": "25110374", "sc": 14, "md5": "da891a7d81f7a5e43b571168cc483b6c dba0b4c99ef37cadfc4bacd61fcefa5b d6b199bfae35246564c598ac78d84c91 38a945eeff5b5a587a26dcc6560e0061 58605af6b50b01f15c0cc3ee2aa75e33 c30566c355ae09ea68673e2940d49d0a 7972c906aef51380310363093e141ef8 dabdd545400415da6d29125bf872"}}
			try:
				arr = json.loads(line.strip().replace(', null,',',0,'))
				eid = int(arr.get('essay_id', 0))
				ver = str(arr.get('version',0))
				rid = arr.get('request_id',0)
				uid = arr.get('user_id',0)
				ct = arr.get('ctime',0)
				
				row = r.get(eid)
				row = json.loads(row) if row else {}
				if not ver in row: 
					print(">> snts missed:", eid, ver, arr['id'], flush=True)
					row[ver] = spacy.snts(arr['essay'])
					r.set(eid, json.dumps(row))

				snts = row[ver] 
				sour = {"rid": int(rid), "uid": int(uid), "sc":len(snts), "md5": sntmd5(snts), "ct": int(ct)}
				actions.append({'_op_type':'index', '_index':idxname,  "_id": f"{eid}-{ver}", "_source": sour})
				if len(actions) > batch : 
					helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
					actions=[]
					print(eid, ver, rid, uid, flush=True) 
			except Exception as ex:
				print(">>callback ex:", ex, line)
		helpers.bulk(client=self.es,actions=actions, raise_on_error=False)
		print("load eevjson finished:", infile,idxname)

	def load_pika_eev(self, ibeg, iend, idxname = 'eevsim', pikahost='192.168.121.3', port=9221, db=2, batch=100000): 
		''' id: [ibeg, iend) , added 2021.12.2 '''
		r = redis.Redis(pikahost, port=port, db=db, decode_responses=True)
		print("index pika started:", ibeg,iend, r)
		if not self.es.indices.exists(idxname): self.es.indices.create(idxname, config)
		#{"_id": "140948871-9", "_source": {"rid": "10", "uid": "25110374", "sc": 14, "md5": "da891a7d81f7a5e43b571168cc483b6c dba0b4c99ef37cadfc4bacd61fcefa5b d6b199bfae35246564c598ac78d84c91 38a945eeff5b5a587a26dcc6560e0061 58605af6b50b01f15c0cc3ee2aa75e33 c30566c355ae09ea68673e2940d49d0a 7972c906aef51380310363093e141ef8 dabdd545400415da6d29125bf872"}}
		for eidv in r.zrangebyscore("eids", ibeg, iend): 
			try:
				arr = eidv.strip().split(':') #149304158:15
				if len(arr) != 2 : continue
				eid,ver = arr 
				snts = json.loads(r.hget(eid,'snts'))[ver]
				self.es.index(index = idxname, id = f"{eid}-{ver}", body = {'rid': int(r.hget(eid,'rid')), 'uid': int(r.hget(eid,'uid')), 'md5': sntmd5(snts), 'sc':len(snts)})
				print (eidv) 
			except Exception as ex:
				print(">>callback ex:", ex, eidv)
		print("load_pika_eev finished:", ibeg, iend,idxname)

	def init(self, idxname):
		''' init a new index '''
		try:
			if self.es.indices.exists(idxname):
				print("already exists", idxname)
				self.es.indices.delete(idxname)
			self.es.indices.create(idxname, config) #, body=snt_mapping
		except Exception as e:
			print("exception,", str(e))
		print(">>finished " + idxname )

	def clear(self, idxname):
		self.es.delete_by_query(idxname, body={"query": {"match_all": {}}})

	def dump(self, idxname):
		''' --- dump index name to JSON file,ie: pigai06 '''
		for a in helpers.scan(self.es,query={"query": {"match_all": {}}}, index=idxname):
			print (a['_id'] + "\t" + json.dumps(a['_source'])) #{'_index': 'pigai06', '_type': '_doc', '_id': '0_40088501_2_0', '_score': None, '_source': {'snt': 'Whether of courses in psychology .', 'postag': 'Whether_IN of_IN courses_NNS in_IN psychology_NN ._.', 'np': ['courses', 'psychology']}, 'sort': [82714]}
		print(">>finished " , idxname, file=sys.stderr)

	def ids(self, idxname):
		''' --- dump _id to file,ie: eev '''
		for a in helpers.scan(self.es,query={"query": {"match_all": {}}}, index=idxname):
			print(a['_id']) 
		print(">>finished " , idxname, file=sys.stderr)

	def load(self, kv_json , idxname):
		''' submit kv_json to es_index, format: _id<TAB><JSON> 20-2-19 '''
		with open(kv_json,'r') as fp:
			actions=[]
			for line in fp.readlines():
				arr = line.strip().split("\t")
				if len(arr) >= 2:
					try:
						d = json.loads(arr[1])
						actions.append({'_op_type':'index', '_index':idxname, '_id':arr[0].strip() , '_source':d})
					except Exception as e:
						print("ex:", e, line)
			helpers.bulk(client=es,actions=actions, raise_on_error=False)
		print(">>finished " , kv_json, idxname )

	def keys(self, idxname):
		''' --- dump index name,ie: pigai06 '''
		query={"query" : {"match_all" : {}}}
		scanResp= helpers.scan(client= es, query=query, scroll= "10m", index= idxname , timeout="10m")
		for resp in scanResp:
			print(resp['_id'])

if __name__ == '__main__':
	fire.Fire(ES)

def test(): #https://elasticsearch-py.readthedocs.io/en/master/
	doc = {
    'author': 'kimchy',
    'text': 'Elasticsearch: cool. bonsai cool.',
    'timestamp': '2020-6-24',}
	res = es.index(index="test-index", id=2, body=doc)
	print("result", res['result'])

	res = es.get(index="test-index", id=1)
	print(res['_source'])

	es.indices.refresh(index="test-index")

	res = es.search(index="test-index", body={"query": {"match_all": {}}})
	print("Got %d Hits:" % res['hits']['total']['value'])
	for hit in res['hits']['hits']:
		print("%(timestamp)s %(author)s: %(text)s" % hit["_source"])


def submit(jsonfile, idxname ='docsim', host='127.0.0.1', id='eid'):
	''' {"eid": "1", "rid": "1419931", "uid": "19704146", "snts": ["Next Monday, a foreign friend will come to meet me.", "I want to give him some representative gifts.", "The 
first thing I want to send is tea.", "The tea culture in Sichuan is very deep.", "On the one hand, Sichuan tea has a strong aroma and a good taste.", "On the other hand
, tea also reflects the relaxed and elegant life culture of Sichuan people.", "The second thing I want to send is the hot pot bottom.", "It is a very symbolic symbol of
 Sichuan.", "Also reflects the food culture of Scihua"]} '''
	print(r, infile, host, idxname, flush=True)
	es = Elasticsearch([host]) 
	file = open(infile, 'r')
	while True:
		line = file.readline()
		if not line: break; 
		try:
			arr = json.loads(line.strip())
			eid = arr[id]
			del arr[id]
			es.index(index=idxname, id=eid, body=arr)
		except Exception as e:# skip those speical keys 
			print("ex:", e)	
	file.close()
	print(">> finished:", infile, flush=True)

def submit_batch(jsonfile, idxname ='sntsim', es_host='127.0.0.1', id='eid', batch=100000):
	from elasticsearch import helpers
	print(r, infile, es_host, idxname, flush=True)
	es = Elasticsearch([es_host]) 
	actions=[]
	file = open(infile, 'r')
	while True:
		line = file.readline()
		if not line: break; 
		try:
			arr = json.loads(line.strip())
			eid = arr[id]
			del arr[id]
			actions.append({'_op_type':'index', '_index':idxname, '_id':eid , '_source':d})
			if len(actions) > 0  : 
				helpers.bulk(client=es,actions=actions, raise_on_error=False)
				actions=[]					
		except Exception as e:# skip those speical keys 
			print("ex:", e)	
	file.close()
	if len(actions) > 0  : 
		helpers.bulk(client=es,actions=actions, raise_on_error=False)
	print(">> finished:", infile, flush=True)


'''
 "mappings": {
    "_source": {
      "includes": [
        "*.count",
        "meta.*"
      ],
      "excludes": [
        "meta.description",
        "meta.other.*"
      ]
    }
  }

	def bulk_submit(self, actions):
		helpers.bulk(client=self.es,actions=actions) #, raise_on_error=False

PUT my_index/_doc/1
{
  "name": "Some binary blob",
  "blob": "U29tZSBiaW5hcnkgYmxvYg==" 
}


PUT maptest/_mapping/test
{
    "dynamic_templates": [
        {
            "en": {
                "match": "*_json",
                "match_mapping_type": "keyword",
                "mapping": {
                    "type": "keyword",
                    "store": "yes",
                    "index": "not_analyzed"
                }
            }
        }
    ]
}

PUT /my_index
{
    "mappings": {
        "my_type": {
            "dynamic_templates": [
                { "es": {
                      "match":              "*_es", 
                      "match_mapping_type": "string",
                      "mapping": {
                          "type":           "string",
                          "analyzer":       "spanish"
                      }
                }},
                { "en": {
                      "match":              "*", 
                      "match_mapping_type": "string",
                      "mapping": {
                          "type":           "string",
                          "analyzer":       "english"
                      }
                }}
            ]
}}}
https://www.elastic.co/guide/cn/elasticsearch/guide/current/custom-dynamic-mapping.html

GET /eevsim/_search
{
  "query": {
    "bool": {
    "must_not":{"term" : {"uid": "123"} },  
    "should": [
        {
          "match": {
            "md5": {
              "query": "201f2e517903766f7c41b90ef27f346f",
              "boost": 2
            }
          }
        },
        {
          "match": {
            "md5": {
              "query": "35179a54ea587953021400eb0cd23201", 
              "boost": 3
            }
          }
        }
      ],
       
      "minimum_should_match": "90%"
    }
  }
}

{"_id": "3-20", "_source": {"rid": 11386, "uid": 21, "sc": 13, "md5": "0f4d9831fee517c2b4223e4ebf6d0930 bfd2f212e4fdbc1b4b9e3c4d14fa42b1 20b30ac261d74904e7b9b3e2cf313728 a0
04c0a2b3e048968cea61002c71b1c6 83f565eac47ede4b022a77cd08d85bc5 a269cf9ee7c15792566a9c4953aed1a6 f380b022b086b477e6f1f6882fcf3b07 fa74e6c1cded62fad1c99999b6d613e0 c4da8d100
6203cb7ba6103ec55c12edc 7af1c8021855d7605e6e7b3d1c370372 d33a962163197dda27c395925a2d1d48 232d21f3b22d6d6fbb6b7fd035e03053 c9dbf1351587627236c0893aac2d1573"}}

12.2 , wrong program's id range 
127.0.0.1:9221[2]> zrevrange ids 0 10
 1) "811997267"
 2) "811997259"
 3) "811997255"
 4) "811997251"
 5) "811997231"
 6) "811997229"
 7) "811997215"
 8) "811997213"
 9) "811997195"
10) "811997191"
11) "811997185"

if spacy: 
	from spacy.lang import en
	spacy.sntbr		= (inst := en.English(), inst.add_pipe("sentencizer"))[0]
	spacy.snts		= lambda essay: [ snt.text.strip() for snt in  spacy.sntbr(essay).sents]
	spacy.nlp		= spacy.load('en_core_web_sm')
	spacy.frombs	= lambda bs: list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
	spacy.tobs		= lambda doc: ( doc_bin:= spacy.tokens.DocBin(), doc_bin.add(doc), doc_bin.to_bytes())[-1]
	spacy.vplem		= lambda doc,ibeg,iend: doc[ibeg].lemma_ + " " + doc[ibeg+1:iend].text.lower()
	spacy.getdoc	= lambda snt: ( bs := redis.bs.get(snt), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), redis.bs.setnx(snt, spacy.tobs(doc)) if not bs else None )[1]
	spacy.sntdocs	= lambda hkey: [spacy.getdoc(snt) for snt in json.loads(redis.r.hget(hkey,'snts'))] #'doc:inau:inau/1893-Cleveland.txt'

2021.12.12
ubuntu@ubuntu:/ftp/esmd5$ nohup python3 -m cikuu.bin.es loadjson eev2021.esmd5 eevsim & 
[1] 47096

 "_doc": {
				  "dynamic_templates": [
					{
					  "strings_as_keywords": { # only map string to keyword, not text 
						"match_mapping_type": "string",
						"mapping": {
						  "type": "keyword"
						}
					  }
					}
				  ]
				},

'''


# 2022.3.3  uvicorn annotate:app --host 0.0.0.0 --port 18000 --reload
import json,requests,hashlib,os
from uvirun import * 
from en import nlp , snts
from en.terms import *
os.dskurl = os.getenv('dskurl',"gpu120.wrask.com:9102")

def stag(snt):
	sdoc = nlp(snt)
	tags = []
	tags.append ("简单句" if len([t for t in sdoc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 else "复杂句" )
	if len([t for t in sdoc if t.dep_ == 'conj' and t.head.dep_ == 'ROOT']) > 0:
		tags.append('并列句')
	return tags 

@app.post('/annotate/tagsnts')
def annotate_tagsnts(snts:list=["I think that I am going to go to the cinema.","The quick fox jumped over the lazy dog."]):  
	''' sentence level tags ''' 
	return [ (snt, stag(snt)) for snt in snts ]

@app.get('/annotate/sntbr')
def annotate_sntbr(text="I think that I am going to go to the cinema. The quick fox jumped over the lazy dog.", trim:bool=True, addtag:bool=True):  
	''' snt break ''' 
	docsnts = snts(text, trim) 
	return docsnts if not addtag else annotate_tagsnts(docsnts)

@app.get('/annotate/spantags')
def annotate_spantags(text="I think that I am going to go to the cinema. The quick fox jumped over the lazy dog.", classid:int=0):  
	''' {"code":0,"data":{"entities":[[16,25,"大小写",1],[16,25,"标点符号",2]]}} ''' 
	doc = nlp(text) 
	spans = [ [doc[np.start].idx, doc[np.start].idx + len(np.text), "NP", classid] for np in doc.noun_chunks if np.end - np.start > 1]
	for name, ibeg,iend in matchers['ap'](doc) :
		spans.append( [doc[ibeg].idx, doc[ibeg].idx + len(doc[ibeg:iend].text), "AP", classid] )
	for name, ibeg,iend in matchers['vp'](doc) :
		spans.append( [doc[ibeg].idx, doc[ibeg].idx + len(doc[ibeg:iend].text), "VP", classid] )
	return {"code":0, "data":{"entities":spans} }

@app.get('/annotate/phrase')
def annotate_phrase(text="I think that I am going to go to the cinema. The quick fox jumped over the lazy dog.", classes:str="NP,VP,AP"):  
	''' {"classes":["NP","NP2"],"annotations":[["Terrible customer service.",{"entities":[[0,17,"NP2"],[18,25,"NP"]]}], ''' 
	doc = nlp(text) 
	spans = [ [doc[np.start].idx, doc[np.start].idx + len(np.text), "NP"] for np in doc.noun_chunks if np.end - np.start > 1]
	for name, ibeg,iend in matchers['ap'](doc) :
		spans.append( [doc[ibeg].idx, doc[ibeg].idx + len(doc[ibeg:iend].text), "AP"] )
	for name, ibeg,iend in matchers['vp'](doc) :
		spans.append( [doc[ibeg].idx, doc[ibeg].idx + len(doc[ibeg:iend].text), "VP"] )
	return {"classes":classes.strip().split(','), "annotations":[[text,{"entities":spans}]] }

@app.get('/annotate/pos')
def annotate_pos(text="I think that I am going to go to the cinema. The quick fox jumped over the lazy dog.", classes:str="VERB,NOUN,ADJ,ADV"):  
	''' {"classes":["NP","NP2"],"annotations":[["Terrible customer service.",{"entities":[[0,17,"NP2"],[18,25,"NP"]]}], ''' 
	doc = nlp(text) 
	pos = classes.strip().split(",")
	spans = [ [t.idx, t.idx + len(t.text), t.pos_] for t in doc if t.pos_ in pos]
	return {"classes":pos,"annotations":[[text,{"entities":spans}]] }

@app.get('/annotate/clause')
def annotate_clause(text="I think that I am going to go to the cinema. What I think is right.", classes:str="ccomp,subcl,csubj,xcomp"):  
	''' {"classes":["NP","NP2"],"annotations":[["Terrible customer service.",{"entities":[[0,17,"NP2"],[18,25,"NP"]]}], ''' 
	doc = nlp(text) 
	spans = []
	for v in [t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT' ] : # non-root
		children = list(v.subtree)
		start = children[0].i  	#end = children[-1].i 
		cl = " ".join([c.text for c in v.subtree])
		spans.append ([doc[start].idx, doc[start].idx + len(cl), v.dep_])
	return {"classes":classes.strip().split(','),"annotations":[[text,{"entities":spans}]] }

@app.get('/annotate/non_pred_verb')
def annotate_non_pred_verb(text="I think that I am going to go to the cinema. It is sunken.", classes:str="vtov,VBN,vvbg"):  
	''' {"classes":["NP","NP2"],"annotations":[["Terrible customer service.",{"entities":[[0,17,"NP2"],[18,25,"NP"]]}], ''' 
	doc = nlp(text) 
	spans = [ [t.idx, t.idx + len(t.text), "VBN"] for t in doc if t.tag_ == 'VBN']
	for name, ibeg,iend in matchers['vtov'](doc) :
		spans.append( [doc[ibeg].idx, doc[ibeg].idx + len(doc[ibeg:iend].text), "vtov"] )
	for name, ibeg,iend in matchers['vvbg'](doc) :
		spans.append( [doc[ibeg].idx, doc[ibeg].idx + len(doc[ibeg:iend].text), "vvbg"] )
	return {"classes":classes.strip().split(','),"annotations":[[text,{"entities":spans}]] }

@app.get('/annotate/stype')
def annotate_stype(text="I think that I am going to go to the cinema. What I think is right."):  #, classes:str="simple,complex,compound"
	''' {"classes":["NP","NP2"],"annotations":[["Terrible customer service.",{"entities":[[0,17,"NP2"],[18,25,"NP"]]}], ''' 
	doc = nlp(text)
	stags = []
	for sent in doc.sents:
		spans = []
		sdoc = sent.as_doc()
		spans.append ("simple" if len([t for t in sdoc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 else "complex" )
		if len([t for t in sdoc if t.dep_ == 'conj' and t.head.dep_ == 'ROOT']) > 0:
			spans.append('compound')
		stags.append( {"start":sent.start, "end":sent.end, "sent": sent.text, "stags": spans })
	return stags
	
feedbacks = lambda dic :  [ v for k,v in dic['feedback'].items() ]
@app.get("/annotate/feedback")
def dsk_feedback(text:str="The quick fox jumped over the lazy dog. The justice delayed is justice denied."):
	''' '''
	dsk = requests.post(f"http://{os.dskurl}/essay/gecdsk", json={"rid":"10", "key": hashlib.md5(text.encode("utf-8")).hexdigest(), "essay":text}).json()
	return  [ ( dic.get('meta',{}).get('snt',''), feedbacks(dic) ) for dic in dsk['snt']]

if __name__ == "__main__":  
	print(annotate_non_pred_verb())


# 2022.2.7
import streamlit as st
import spacy
import pandas as pd

nlp			= spacy.load('en_core_web_sm')
merge_nps	= nlp.create_pipe("merge_noun_chunks")
postag		= lambda snt: pd.DataFrame([ (t.text, t.tag_) for t in nlp(snt)], columns=['word','pos'])
tokenize	= lambda snt: " ".join([t.text for t in nlp(snt) if len(t.text.strip())]).strip()

def parse(snt, merge_np= False):
	doc = nlp(snt)
	if merge_np : merge_nps(doc)
	return pd.DataFrame({'word': [t.text for t in doc], 'tag': [t.tag_ for t in doc],'pos': [t.pos_ for t in doc],'head': [t.head.orth_ for t in doc],'dep': [t.dep_ for t in doc], 'lemma': [t.text.lower() if t.lemma_ == '-PRON-' else t.lemma_ for t in doc],
	'n_lefts': [ t.n_lefts for t in doc], 'left_edge': [ t.left_edge.text for t in doc], 
	'n_rights': [ t.n_rights for t in doc], 'right_edge': [ t.right_edge.text for t in doc],
	}) #'subtree': str([ list(t.subtree) for t in doc]),'children': str([ list(t.children) for t in doc]),

st.title("Sentence parsing")
snt = st.text_input("Input a sentence", "The quick fox jumped over the lazy dog.")
merge_np = st.checkbox('merge NP', value=False)
if st.button("submit"):
	df = parse(snt, merge_np)
	st.write(df)



# 2022.2.21
import json, sys, time, fire, os

def run(infile):
	with open("frame.json", 'a+') as fw:
		try:
			arr = [json.loads(line.strip()) for line in open(infile,'r').readlines() ]
			word = infile.split('.')[0]
			fw.write(f"{word}\t{json.dumps(arr)}\n")
		except Exception as e:
			print("ex:", e, infile)

if __name__ == '__main__':
	fire.Fire(run)



# redis rest wrapper, added 2022.3.2,  backend is redis/pika/kvrocks 
import json,fire,os,redis,requests
from uvirun import * 

@app.get("/redis/{db}")
def redis_execute(db:int=0, code:str="r.info()"):
	''' one line python code runner, '''
	if not hasattr(redis, code): setattr(redis, code, compile(code, '', 'eval') )  
	return eval(getattr(redis, code), {"requests":requests, "json":json, "r":redis.dbs[db], 'redis':redis})

@app.get("/redis/{db}/hjoin/{hkey}")
def hgetall_join(hkey:str="rid:2573411", db:int=0, sepa:str="-"):
	''' eidv:  [{k}-{v}] '''
	return [f"{k}{sepa}{v}" for k,v in redis.dbs[db].hgetall(hkey).items()]

#-----  dsk func --- 
@app.get("/dsk/{rid}/eidvlist")
def eidv_list(rid: int=2573411): return [f"{k}-{v}" for k,v in redis.dbs[0].hgetall(f"rid:{rid}").items()]
@app.get("/dsk/{rid}/snts")
def rid_snts(rid: int=2573411): return (snts := [], [ snts.extend(json.loads(redis.dbs[0].hget(eidv, 'snts'))) for eidv in eidv_list(rid) ] )[0]
@app.get("/dsk/{rid}/mkfs")
def rid_mkfs(rid: int=2573411): return [	json.loads(mkf) for mkf in redis.dbs[1].mget( rid_snts(rid)) ]
@app.get("/dsk/{eidv}/score")
def eidv_score(eidv:str='2573411-2'): return json.loads(redis.dbs[0].hget(eidv,'dsk')).get('info',{}).get('final_score',0.0)

def run(self, port, rhost='127.0.0.1', rport=6379, dbsum=1, bsdb=-1) :
	''' open redis connections , ie:  python -m uvirun.kvrocks 16379
	when bsdb >=0 : enable spacy byte stream cache 
	'''
	redis.dbs	= [redis.Redis(rhost, port=rport, db=i, decode_responses=True) for i in range(0,dbsum) ]
	#redis.r		= redis.dbs[0]  # default 
	if bsdb >=0 : 
		redis.bs	= redis.Redis(rhost, port=rport, db=bsdb, decode_responses=False)
		import spacy 
		if not hasattr(spacy, 'nlp'): 
			spacy.nlp		= spacy.load('en_core_web_sm')
			spacy.frombs	= lambda bs: list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
			spacy.tobs		= lambda doc: ( doc_bin:= spacy.tokens.DocBin(), doc_bin.add(doc), doc_bin.to_bytes())[-1]
			spacy.getdoc	= lambda snt: ( bs := redis.bs.get(snt), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), redis.bs.setnx(snt, spacy.tobs(doc)) if not bs else None )[1]

	uvicorn.run(app, host='0.0.0.0', port=port)

if __name__ == "__main__":  
	fire.Fire(run)

'''
eidv_list   = lambda rid: [f"{k}-{v}" for k,v in redis.dsk.hgetall(f"rid:{rid}").items()]
rid_snts	= lambda rid: (	snts := [], [ snts.extend(json.loads(redis.dsk.hget(eidv, 'snts'))) for eidv in eidv_list(rid) ] )[0]
rid_mkfs	= lambda rid: [	json.loads(mkf) for mkf in redis.mkf.mget( rid_snts(rid)) ]
eidv_score  = lambda eidv: json.loads(redis.dsk.hget(eidv,'dsk')).get('info',{}).get('final_score',0.0)
'''


# 2022-2-17  
import json,fire,sys, os 
from dic import * 

class util(object):

	def __init__(self):  pass

	def snt(self, name): 
		''' gzjc/clec/gaokao '''
		lines = readzip(name)
		[ print(line.strip()) for line in lines ]


if __name__ == '__main__':
	fire.Fire(util)



import requests 
from collections import	Counter, defaultdict

import hashlib
sntmd5	= lambda sntarr: " ".join([hashlib.md5(snt.strip().lower().encode("utf-8")).hexdigest() for snt in sntarr if len(snt) > 1])

from math import log as ln
def likelihood(a,b,c,d, minus=None):  #from: http://ucrel.lancs.ac.uk/llwizard.html
	try:
		if a is None or a <= 0 : a = 0.000001
		if b is None or b <= 0 : b = 0.000001
		E1 = c * (a + b) / (c + d)
		E2 = d * (a + b) / (c + d)
		G2 = round(2 * ((a * ln(a / E1)) + (b * ln(b / E2))), 2)
		if minus or  (minus is None and a/c < b/d): G2 = 0 - G2
		return G2
	except Exception as e:
		print ("likelihood ex:",e, a,b,c,d)
		return 0




# https://github.com/asehmi/simple-streamlit-fastapi-integration/blob/main/lrp_app.py
import os
import time
import streamlit as st
import requests

# --------------------------------------------------------------------------------

API_HOST='localhost'
API_PORT=8000
API_BASE_URL='http://localhost:8000'

from utils import SessionState
# Session State variables:
state = SessionState.get(
    API_APP = None,
    API_STARTED=False,
)

# --------------------------------------------------------------------------------

# NOTE: Design point... only main() is allowed to mutate state. All supporting functions should not mutate state.
def main():
    st.title('LR Process Manager')

    # RUN LRP
    if not state.API_STARTED:
        st.write('To launch your LRP click the button below.')
        if st.button('\U0001F680 Launch'):

            import subprocess
            import threading

            def run(job):
                print (f"\nRunning job: {job}\n")
                proc = subprocess.Popen(job)
                proc.wait()
                return proc

            job = ['python', os.path.join('./', 'lrp_bootstrapper.py'), API_HOST, str(API_PORT)]

            # server thread will remain active as long as streamlit thread is running, or is manually shutdown
            thread = threading.Thread(name='FastAPI-LRP-Bootstrapper', target=run, args=(job,), daemon=True)
            thread.start()

            time.sleep(2)

            # !! Start the LRP !!
            requests.get(f'{API_BASE_URL}/run')

            state.API_STARTED = True

            st.experimental_rerun()

    if state.API_STARTED:
        st.markdown(f'''
            The LRP API is running. If you\'d like to terminate the LRP click the button below.
            ### API docs
            - [**http://{API_HOST}:{API_PORT}/docs**](http://{API_HOST}:{API_PORT}/docs)
            - [**http://{API_HOST}:{API_PORT}/redoc**](http://{API_HOST}:{API_PORT}/redoc)
        ''')

        if st.button('\U0001F525 Shutdown LRP'):
            requests.get(f'{API_BASE_URL}/shutdown')

            state.API_STARTED = False

            st.experimental_rerun()

def sidebar():
    # ABOUT
    st.sidebar.header('About')
    st.sidebar.info('FastAPI Wrapper to run and stop a LRP!\n\n' + \
        '(c) 2021. Oxford Economics Ltd. All rights reserved.')
    st.sidebar.markdown('---')


if __name__ == '__main__':
    main()
    sidebar()


# 2022.2.3
import streamlit as st
import pandas as pd
import os
from func import * 

app_state = st.experimental_get_query_params()
if not isinstance(app_state, str): app_state = {k: v[0] if isinstance(v, list) else v for k, v in app_state.items()} 

if "f" in app_state: 
	f = app_state['f']
	x = __import__(f"func.{f}", fromlist=['run'])
	x.run()
else: 
	st.title("Func list")
	for root, dirs, files in os.walk("func"):
		for file in files: 
			if file.endswith(".py") and not file.startswith("_") and not 'common' in file:
				file = file.split(".")[0]
				st.write(f"[{file}](?f={file})")




import textdistance
from nltk.tokenize import TreebankWordTokenizer
tokenizer = TreebankWordTokenizer()
hamming = textdistance.Hamming(external=False)
edits = lambda snt0="I like it.", snt1="I love it.": hamming(tokenizer.tokenize(snt0), tokenizer.tokenize(snt1))

def sent_diff(snt:str="I like it.", refsnt:str="I love it."):
	src_toks = tokenizer.tokenize(snt)
	ref_toks = tokenizer.tokenize(refsnt)
	edits = hamming(src_toks, ref_toks)
	return 1 - round(edits / len(ref_toks), 2) 

print (sent_diff())


word_gsl2 = {
"abroad",
"absence",
"absences",
"absent",
"absolute",
"absolutely",
"accident",
"accidents",
"accidental",
"accidentally",
"accuse",
"accusing",
"accuses",
"accused",
"accustom",
"accustoms",
"accustoming",
"accustomed",
"ache",
"aching",
"aches",
"ached",
"admire",
"admiring",
"admires",
"admired",
"admirable",
"admiration",
"advertise",
"advertising",
"advertises",
"advertiser",
"advertised",
"advertisement",
"advertisements",
"advice",
"advising",
"advises",
"advisers",
"adviser",
"advised",
"advise",
"advisory",
"aeroplane",
"aeroplanes",
"afford",
"afforded",
"affording",
"affords",
"affordable",
"afraid",
"afternoon",
"afternoons",
"agriculture",
"agricultural",
"ahead",
"aim",
"aims",
"aiming",
"aimed",
"aimless",
"aimlessness",
"airplane",
"airplanes",
"alike",
"alive",
"aloud",
"altogether",
"ambition",
"ambitions",
"ambitious",
"ambitiously",
"amuse",
"amusing",
"amuses",
"amused",
"amusement",
"amusements",
"anger",
"angers",
"angering",
"angered",
"angry",
"angrily",
"angrier",
"angriest",
"angle",
"angles",
"angling",
"angled",
"annoy",
"annoys",
"annoying",
"annoyed",
"annoyance",
"anxiety",
"anxieties",
"anxious",
"anxiously",
"apart",
"apologize",
"apologizes",
"apologized",
"apologise",
"apologising",
"apologises",
"apologised",
"apology",
"apologies",
"apologetic",
"applaud",
"applauds",
"applauding",
"applauded",
"applause",
"apple",
"apples",
"approve",
"approving",
"approves",
"approved",
"approval",
"disapprove",
"disapproving",
"disapproves",
"disapproved",
"disapproval",
"arch",
"arching",
"arches",
"arched",
"argue",
"arguing",
"argues",
"argued",
"arguable",
"argument",
"arguments",
"arrange",
"arranged",
"arranging",
"arranges",
"arrangement",
"arrangements",
"arrest",
"arrests",
"arresting",
"arrested",
"arrow",
"arrows",
"artificial",
"artificially",
"ash",
"ashes",
"ashamed",
"ashamedly",
"aside",
"asleep",
"astonish",
"astonishing",
"astonishes",
"astonished",
"astonishment",
"attend",
"attends",
"attending",
"attended",
"attendant",
"attendance",
"attendances",
"attention",
"attract",
"attracts",
"attracting",
"attracted",
"attractive",
"attraction",
"attractions",
"attractively",
"attractiveness",
"audience",
"audiences",
"aunt",
"aunts",
"autumn",
"avenue",
"avenues",
"avoid",
"avoids",
"avoiding",
"avoided",
"avoidance",
"avoidable",
"unavoidable",
"awake",
"awaken",
"awoken",
"awkward",
"awkwardly",
"axe",
"axing",
"axes",
"axed",
"ax",
"baby",
"babies",
"bag",
"bags",
"bagging",
"bagged",
"baggage",
"bake",
"baking",
"bakes",
"bakers",
"baker",
"baked",
"balance",
"balancing",
"balances",
"balanced",
"unbalanced",
"band",
"banded",
"banding",
"bands",
"barber",
"barbers",
"bare",
"baring",
"bares",
"bared",
"barely",
"bargain",
"bargains",
"bargaining",
"bargained",
"barrel",
"barrels",
"basin",
"basins",
"basket",
"baskets",
"bath",
"baths",
"bathe",
"bathing",
"bathes",
"bathed",
"bay",
"bays",
"beak",
"beaks",
"beaked",
"beam",
"beams",
"beamed",
"beaming",
"bean",
"beans",
"beard",
"beards",
"bearded",
"beast",
"beasts",
"beat",
"beats",
"beating",
"beaten",
"beg",
"begs",
"begging",
"begged",
"beggars",
"beggar",
"behave",
"behaves",
"behaved",
"behaving",
"behaviour",
"behaviours",
"behaviors",
"behavioural",
"behavior",
"behavioral",
"bell",
"bells",
"belt",
"belts",
"belting",
"belted",
"bend",
"bent",
"bends",
"bending",
"berry",
"berries",
"bicycle",
"bicycling",
"bicycles",
"bicycled",
"billion",
"billions",
"bind",
"binds",
"binding",
"binders",
"binder",
"birth",
"births",
"birthdays",
"birthday",
"bit",
"bits",
"bite",
"bites",
"biting",
"bitten",
"bitter",
"bitterness",
"bitterly",
"bitterest",
"bitterer",
"blade",
"blades",
"blame",
"blaming",
"blames",
"blamed",
"blameless",
"bless",
"blessed",
"blesses",
"blessing",
"blessings",
"blind",
"blinds",
"blindly",
"blinding",
"blinded",
"blindness",
"block",
"blocks",
"blocking",
"blocked",
"boast",
"boasts",
"boasting",
"boasted",
"boastful",
"boil",
"boils",
"boiling",
"boilers",
"boiler",
"boiled",
"bold",
"boldness",
"boldly",
"boldest",
"bolder",
"bone",
"boning",
"bones",
"boned",
"border",
"borders",
"bordering",
"bordered",
"borrow",
"borrows",
"borrowing",
"borrowed",
"bottle",
"bottling",
"bottles",
"bottled",
"bottom",
"bottoms",
"bound",
"bounds",
"bounding",
"bounded",
"boundless",
"boundary",
"boundaries",
"bow",
"bows",
"bowing",
"bowed",
"bowl",
"bowls",
"bowling",
"bowled",
"brain",
"brains",
"brass",
"brave",
"braving",
"braves",
"bravely",
"braved",
"bravery",
"breakfast",
"breakfasts",
"breakfasting",
"breakfasted",
"breath",
"breaths",
"breathlessly",
"breathless",
"breathe",
"breathing",
"breathes",
"breathed",
"bribe",
"bribing",
"bribes",
"bribed",
"bribery",
"brick",
"bricks",
"broadcast",
"broadcasting",
"broadcasts",
"brown",
"browns",
"browning",
"browned",
"brush",
"brushing",
"brushes",
"brushed",
"bucket",
"buckets",
"bunch",
"bunches",
"bundle",
"bundling",
"bundles",
"bundled",
"burst",
"bursts",
"bursting",
"bury",
"burying",
"buries",
"buried",
"burial",
"burials",
"bus",
"buses",
"bussed",
"bush",
"bushes",
"bushy",
"busy",
"busily",
"busiest",
"busier",
"butter",
"butters",
"buttering",
"buttered",
"button",
"buttons",
"buttoning",
"buttoned",
"unbuttoned",
"cage",
"caging",
"cages",
"caged",
"cake",
"cakes",
"calculate",
"calculator",
"calculating",
"calculates",
"calculated",
"calculation",
"calculations",
"calm",
"calms",
"calmly",
"calming",
"calmed",
"calmness",
"camera",
"cameras",
"camp",
"camped",
"camping",
"camps",
"canal",
"canals",
"cap",
"caps",
"capping",
"capped",
"cape",
"capes",
"caped",
"card",
"cards",
"carriage",
"carriages",
"cart",
"carts",
"carting",
"carted",
"cat",
"cats",
"cattle",
"caution",
"cautions",
"cautioning",
"cautioned",
"cautious",
"cautiously",
"cave",
"caving",
"caves",
"caved",
"cent",
"cents",
"centimetre",
"centimetres",
"century",
"centuries",
"ceremony",
"ceremonies",
"ceremonial",
"chain",
"chains",
"chaining",
"chained",
"chair",
"chairs",
"chairman",
"chalk",
"chalks",
"charm",
"charms",
"charming",
"charmed",
"cheap",
"cheapness",
"cheaply",
"cheapest",
"cheaper",
"cheat",
"cheats",
"cheating",
"cheated",
"check",
"checks",
"checking",
"checked",
"cheer",
"cheery",
"cheers",
"cheering",
"cheerful",
"cheered",
"cheese",
"cheeses",
"cheque",
"cheques",
"chest",
"chests",
"chicken",
"chickens",
"chimney",
"chimneys",
"christmas",
"christmases",
"civilise",
"civilizing",
"civilizes",
"civilized",
"civilize",
"civilising",
"civilises",
"civilised",
"civilisation",
"civilisations",
"clay",
"clays",
"clean",
"cleaned",
"cleaner",
"cleaners",
"cleanest",
"cleaning",
"cleanly",
"cleans",
"cleanness",
"clerk",
"clerks",
"clever",
"cleverness",
"cleverly",
"cleverest",
"cleverer",
"cliff",
"cliffs",
"climb",
"climbs",
"climbing",
"climbers",
"climber",
"climbed",
"clock",
"clocks",
"cloth",
"cloths",
"clothing",
"clothes",
"club",
"clubbed",
"clubbing",
"clubs",
"coarse",
"coarsest",
"coarser",
"coarseness",
"coarsely",
"coat",
"coats",
"coffee",
"coffees",
"collar",
"collars",
"collect",
"collects",
"collecting",
"collected",
"collective",
"collectively",
"collector",
"collectors",
"collection",
"collections",
"comb",
"combs",
"combing",
"combed",
"combine",
"combining",
"combines",
"combined",
"combination",
"combinations",
"comfort",
"uncomfortable",
"discomfort",
"comforts",
"comforting",
"comforted",
"comfortably",
"comfortable",
"commerce",
"commercial",
"commercially",
"companion",
"companions",
"companionship",
"compare",
"comparable",
"compared",
"compares",
"comparative",
"comparison",
"comparisons",
"comparing",
"comparatively",
"compete",
"competing",
"competes",
"competed",
"competition",
"competitions",
"complain",
"complaints",
"complaint",
"complains",
"complaining",
"complained",
"complicate",
"complicated",
"complicates",
"complicating",
"complication",
"complications",
"compose",
"composing",
"composes",
"composer",
"composed",
"composite",
"composition",
"confess",
"confessing",
"confesses",
"confessed",
"confession",
"confessions",
"confidence",
"confidences",
"confident",
"confidently",
"confuse",
"confusing",
"confuses",
"confused",
"confusion",
"congratulate",
"congratulating",
"congratulates",
"congratulated",
"congratulations",
"connect",
"connected",
"connecting",
"connects",
"connection",
"connections",
"conquer",
"conquers",
"conquerors",
"conqueror",
"conquering",
"conquered",
"conquest",
"conquests",
"conscience",
"consciences",
"conscious",
"consciousness",
"unconscious",
"unconsciously",
"consciously",
"convenience",
"conveniences",
"convenient",
"conveniently",
"conversation",
"conversations",
"cook",
"cooks",
"cooking",
"cookers",
"cooker",
"cooked",
"cool",
"cools",
"coolness",
"coolly",
"cooling",
"coolest",
"cooler",
"cooled",
"copper",
"copy",
"copying",
"copies",
"copied",
"cork",
"corks",
"corking",
"corked",
"corner",
"corners",
"cornered",
"cornering",
"correct",
"corrects",
"correctly",
"correcting",
"corrected",
"correctness",
"correction",
"corrections",
"incorrect",
"incorrectly",
"cottage",
"cottages",
"cough",
"coughs",
"coughing",
"coughed",
"courage",
"courageous",
"cousin",
"cousins",
"cow",
"cows",
"coward",
"cowards",
"cowardly",
"cowardice",
"crack",
"cracks",
"cracking",
"cracked",
"crash",
"crashing",
"crashes",
"crashed",
"cream",
"creams",
"creaming",
"creamed",
"creature",
"creatures",
"creep",
"crept",
"creeps",
"creeping",
"crime",
"crimes",
"criminal",
"criminally",
"critic",
"critics",
"critically",
"critical",
"crop",
"crops",
"cropping",
"cropped",
"cruel",
"cruelly",
"cruelness",
"cruelty",
"cruelties",
"crush",
"crushing",
"crushes",
"crushed",
"cultivate",
"cultivating",
"cultivates",
"cultivated",
"cultivation",
"cup",
"cups",
"cupboards",
"cupboard",
"cure",
"curing",
"cures",
"cured",
"curable",
"incurable",
"curious",
"curiously",
"curiosity",
"curl",
"curls",
"curling",
"curled",
"curly",
"curse",
"cursing",
"curses",
"cursed",
"curtain",
"curtains",
"curve",
"curving",
"curves",
"curved",
"cushion",
"cushions",
"cushioned",
"cushioning",
"custom",
"customs",
"customary",
"customer",
"customers",
"damage",
"damaging",
"damages",
"damaged",
"undamaged",
"damp",
"dampness",
"dampest",
"damper",
"dance",
"danced",
"dancer",
"dancers",
"dances",
"dancing",
"dare",
"dared",
"dares",
"daring",
"deaf",
"deafness",
"deafest",
"deafer",
"debt",
"debts",
"decay",
"decays",
"decaying",
"decayed",
"deceive",
"deceiving",
"deceives",
"deceived",
"deception",
"decrease",
"decreasing",
"decreases",
"decreased",
"deed",
"deeds",
"deer",
"defend",
"defence",
"defends",
"defending",
"defenders",
"defender",
"defended",
"delay",
"delays",
"delaying",
"delayed",
"delicate",
"delicately",
"delicacy",
"delicacies",
"delight",
"delights",
"delighting",
"delightfully",
"delightful",
"delighted",
"deliver",
"delivered",
"deliveries",
"delivering",
"delivers",
"undelivered",
"delivery",
"descend",
"descends",
"descending",
"descended",
"deserve",
"deserving",
"deserves",
"deservedly",
"deserved",
"desk",
"desks",
"despair",
"despairs",
"despairingly",
"despairing",
"despaired",
"devil",
"devils",
"diamond",
"diamonds",
"dictionary",
"dictionaries",
"dig",
"dug",
"digs",
"digging",
"dinner",
"dinners",
"dining",
"dines",
"dined",
"dine",
"dip",
"dips",
"dipping",
"dipped",
"dirt",
"dirty",
"disappoint",
"disappoints",
"disappointing",
"disappointed",
"disappointment",
"disappointments",
"discipline",
"disciplining",
"disciplines",
"disciplined",
"disciplinary",
"discuss",
"discussing",
"discusses",
"discussed",
"discussion",
"discussions",
"disease",
"diseases",
"diseased",
"disgust",
"disgusts",
"disgusting",
"disgusted",
"dish",
"dishes",
"dismiss",
"dismissing",
"dismisses",
"dismissed",
"dismissive",
"dismissively",
"disturb",
"disturbs",
"disturbing",
"disturbed",
"disturbance",
"disturbances",
"ditch",
"ditching",
"ditches",
"ditched",
"dive",
"diving",
"dives",
"dived",
"dove",
"diver",
"divers",
"donkey",
"donkeys",
"dot",
"dots",
"double",
"doubled",
"doubles",
"doubling",
"dozen",
"dozens",
"drag",
"drags",
"dragging",
"dragged",
"drawer",
"drawers",
"drown",
"drowns",
"drowning",
"drowned",
"drum",
"drums",
"drumming",
"drummed",
"duck",
"ducks",
"ducking",
"ducked",
"dull",
"dullness",
"dullest",
"duller",
"during",
"dust",
"dusts",
"dusting",
"dusted",
"duster",
"dusters",
"eager",
"eagerness",
"eagerly",
"earn",
"earns",
"earning",
"earned",
"earnings",
"earnest",
"earnestly",
"ease",
"easing",
"eases",
"eased",
"unease",
"uneasy",
"edge",
"edged",
"edges",
"edging",
"educate",
"educating",
"educates",
"educated",
"educator",
"educators",
"uneducated",
"education",
"educational",
"elastic",
"elastics",
"elasticity",
"elder",
"elderly",
"electric",
"electricity",
"electricians",
"electrician",
"electrical",
"electrically",
"elephant",
"elephants",
"empty",
"emptying",
"empties",
"emptied",
"enclose",
"enclosing",
"encloses",
"enclosed",
"enclosure",
"encourage",
"encouragingly",
"encouraging",
"encourages",
"encouraged",
"encouragement",
"engine",
"engines",
"engineer",
"engineering",
"engineers",
"entertain",
"entertains",
"entertaining",
"entertained",
"entertainment",
"entire",
"entirely",
"envelope",
"envelopes",
"envy",
"envying",
"envies",
"envied",
"envious",
"especial",
"especially",
"essence",
"essences",
"essential",
"essentials",
"essentially",
"evil",
"eviler",
"evilest",
"evilly",
"evilness",
"evils",
"exact",
"exactly",
"examining",
"examines",
"examined",
"examine",
"examiner",
"examiners",
"examination",
"examinations",
"excellent",
"excellence",
"excess",
"excessive",
"excessively",
"excite",
"exciting",
"excites",
"excitedly",
"excited",
"excitement",
"excuse",
"excusing",
"excuses",
"excused",
"explode",
"exploding",
"explodes",
"exploded",
"explosion",
"explosions",
"explosive",
"explore",
"exploring",
"exploration",
"explores",
"explored",
"exploratory",
"explorer",
"explorers",
"explorations",
"extra",
"extras",
"extraordinary",
"extraordinarily",
"extreme",
"extremely",
"fade",
"fading",
"fades",
"faded",
"faint",
"faints",
"faintly",
"fainting",
"fainter",
"fainted",
"faintness",
"false",
"falsely",
"falsehood",
"fan",
"fans",
"fanning",
"fanned",
"fancy",
"fancied",
"fancier",
"fancies",
"fanciest",
"fancily",
"fanciness",
"fancying",
"fashion",
"fashionable",
"fashioned",
"fashioning",
"fashions",
"fasten",
"fastened",
"fastener",
"fasteners",
"fastening",
"fastens",
"fat",
"fats",
"fatten",
"fatty",
"fate",
"fates",
"fault",
"faulty",
"faults",
"feast",
"feasts",
"feasting",
"feasted",
"feather",
"feathers",
"feathering",
"feathered",
"female",
"females",
"fence",
"fencing",
"fences",
"fenced",
"fever",
"feverish",
"fevers",
"fierce",
"fiercest",
"fiercer",
"fierceness",
"fiercely",
"film",
"films",
"filming",
"filmed",
"finger",
"fingers",
"firm",
"firms",
"firmness",
"firmly",
"firmest",
"flag",
"flags",
"flame",
"flames",
"flaming",
"flash",
"flashing",
"flashes",
"flashed",
"flat",
"flats",
"flatten",
"flattening",
"flavour",
"flavours",
"flavouring",
"flavoured",
"flavor",
"flavors",
"flavoring",
"flavored",
"flesh",
"float",
"floats",
"floating",
"floated",
"flood",
"floods",
"flooding",
"flooded",
"flour",
"fold",
"folds",
"folding",
"folded",
"fond",
"fondness",
"fondly",
"fondest",
"fonder",
"fool",
"fools",
"foolishness",
"foolishly",
"foolish",
"fooling",
"fooled",
"foot",
"feet",
"football",
"footballs",
"forbid",
"forbids",
"forbidding",
"forbidden",
"forgive",
"forgiving",
"forgives",
"forgiven",
"forgave",
"fork",
"forks",
"formal",
"formally",
"formality",
"forward",
"forwards",
"frame",
"framing",
"frames",
"framed",
"freeze",
"frozen",
"freezing",
"freezes",
"frequent",
"frequents",
"frequently",
"frequenting",
"frequented",
"frequency",
"infrequent",
"infrequently",
"fright",
"frights",
"frightful",
"frightens",
"frightening",
"frightened",
"frighten",
"fruit",
"fruits",
"fry",
"frying",
"fries",
"fried",
"fun",
"funny",
"funniest",
"funnier",
"funeral",
"funerals",
"fur",
"furs",
"gallon",
"gallons",
"gap",
"gaps",
"garage",
"garages",
"gay",
"gayness",
"gayest",
"gayer",
"gaily",
"gaiety",
"generous",
"generously",
"generosity",
"glory",
"glories",
"glorious",
"goat",
"goats",
"govern",
"govt",
"governs",
"governor",
"governing",
"governed",
"government",
"governments",
"governmental",
"grace",
"gracing",
"graces",
"gracefully",
"graceful",
"graced",
"gradual",
"gradually",
"grain",
"grains",
"gram",
"grams",
"grammar",
"grammars",
"grand",
"grandsons",
"grandson",
"grandparents",
"grandparent",
"grandpa",
"grandmother",
"grandma",
"grandfather",
"granddaughters",
"granddaughter",
"grandchildren",
"grandchild",
"grass",
"grasses",
"grassy",
"grateful",
"gratefully",
"grave",
"graves",
"grease",
"greasing",
"greases",
"greased",
"greasy",
"greed",
"greedy",
"greedily",
"greet",
"greetings",
"greets",
"greeting",
"greeted",
"grey",
"grind",
"grinds",
"grinding",
"guard",
"guarded",
"guarding",
"guards",
"guess",
"guessing",
"guesses",
"guessed",
"guest",
"guests",
"guide",
"guiding",
"guides",
"guided",
"guilty",
"guiltily",
"guilt",
"guiltless",
"gun",
"guns",
"gunner",
"habit",
"habits",
"habitual",
"habitually",
"hair",
"hairy",
"hairs",
"hall",
"halls",
"hammer",
"hammers",
"hammering",
"hammered",
"handkerchief",
"handkerchiefs",
"handle",
"handled",
"handles",
"handling",
"harbor",
"harbours",
"harbour",
"harbors",
"harm",
"harms",
"harmless",
"harming",
"harmful",
"harmed",
"harvest",
"harvests",
"harvesting",
"harvester",
"harvested",
"haste",
"hastily",
"hastens",
"hastening",
"hastened",
"hasten",
"hat",
"hats",
"hate",
"hatred",
"hating",
"hates",
"hated",
"hay",
"heal",
"heals",
"healing",
"healed",
"health",
"healthier",
"healthiest",
"healthily",
"healthy",
"heap",
"heaps",
"heaping",
"heaped",
"hesitate",
"hesitating",
"hesitates",
"hesitated",
"hesitation",
"hesitations",
"hide",
"hid",
"hidden",
"hides",
"hiding",
"hinder",
"hinders",
"hindering",
"hindered",
"hire",
"hiring",
"hires",
"hired",
"hit",
"hitting",
"hitter",
"hits",
"hole",
"holes",
"holiday",
"holidays",
"hollow",
"hollows",
"hollowing",
"hollowed",
"holy",
"holiness",
"holiest",
"holier",
"honest",
"honesty",
"honestly",
"dishonest",
"hook",
"hooks",
"hooking",
"hooked",
"horizon",
"horizons",
"hospital",
"hospitals",
"host",
"hosts",
"hosting",
"hosted",
"hotel",
"hotels",
"hullo",
"hello",
"hey",
"hi",
"humble",
"humbly",
"humbling",
"humbles",
"humbled",
"hunger",
"hungry",
"hungrily",
"hungers",
"hungering",
"hunt",
"hunts",
"hunting",
"hunters",
"hunter",
"hunted",
"hurrah",
"hurry",
"hurrying",
"hurries",
"hurriedly",
"hurried",
"hurt",
"hurts",
"hurting",
"hurtful",
"hut",
"huts",
"ice",
"icy",
"iced",
"ideal",
"ideals",
"ideally",
"idle",
"idling",
"idles",
"idled",
"idly",
"imagine",
"imagining",
"imagines",
"imagined",
"imagination",
"imaginations",
"imaginative",
"imaginary",
"imitate",
"imitating",
"imitates",
"imitated",
"imitation",
"imitations",
"immediate",
"immediacy",
"immediately",
"immense",
"immensely",
"improve",
"improving",
"improves",
"improved",
"improvement",
"improvements",
"inform",
"informs",
"informing",
"informed",
"information",
"informal",
"informally",
"ink",
"inks",
"inky",
"inn",
"inns",
"inquire",
"inquiry",
"inquiring",
"inquiries",
"inquires",
"inquired",
"enquire",
"enquiry",
"enquiring",
"insect",
"insects",
"inside",
"insider",
"insiders",
"instant",
"instantly",
"instrument",
"instruments",
"instrumental",
"insult",
"insults",
"insulting",
"insulted",
"insure",
"insuring",
"insures",
"insured",
"insurance",
"intend",
"intended",
"intending",
"intends",
"intent",
"intently",
"intention",
"intentions",
"intentional",
"intentionally",
"interfere",
"interfering",
"interferes",
"interfered",
"interference",
"international",
"internationally",
"interrupt",
"interrupts",
"interrupting",
"interrupted",
"invent",
"invents",
"inventors",
"inventor",
"inventing",
"invented",
"invention",
"inventions",
"invite",
"inviting",
"invites",
"invited",
"invitation",
"invitations",
"inward",
"inwardly",
"island",
"islands",
"jaw",
"jaws",
"jealous",
"jealously",
"jealousy",
"jewel",
"jewels",
"jewellers",
"jeweller",
"jewellery",
"joke",
"jokes",
"joked",
"joking",
"journey",
"journeys",
"journeying",
"journeyed",
"juice",
"juices",
"juicy",
"jump",
"jumps",
"jumping",
"jumped",
"key",
"keys",
"kick",
"kicks",
"kicking",
"kicked",
"kilogram",
"kilograms",
"kilometre",
"kilometres",
"kiss",
"kissing",
"kisses",
"kissed",
"kitchen",
"kitchens",
"knee",
"knees",
"kneel",
"kneels",
"kneeling",
"knife",
"knives",
"knock",
"knocks",
"knocking",
"knocked",
"knot",
"knotting",
"knotted",
"knots",
"ladder",
"ladders",
"lamp",
"lamps",
"lazy",
"laziness",
"lazily",
"laziest",
"lazier",
"leaf",
"leafy",
"lean",
"leans",
"leaning",
"leaned",
"leather",
"leathers",
"leg",
"legs",
"lend",
"lent",
"lends",
"lending",
"lesson",
"lessons",
"liberty",
"liberties",
"lid",
"lids",
"limb",
"limbs",
"liquid",
"liquids",
"list",
"unlisted",
"lists",
"listed",
"listing",
"litre",
"litres",
"load",
"unloads",
"unloading",
"unloaded",
"loads",
"loading",
"loaded",
"loaf",
"loaves",
"loan",
"loans",
"lock",
"unlocks",
"unlocking",
"unlocked",
"unlock",
"locks",
"locking",
"locked",
"lodging",
"lodgings",
"log",
"logs",
"lone",
"lonely",
"loneliness",
"loose",
"loosely",
"loosen",
"lot",
"lots",
"loud",
"loudness",
"loudly",
"loudest",
"louder",
"loyal",
"loyalty",
"loyally",
"luck",
"lucky",
"luckily",
"luckiest",
"luckier",
"unlucky",
"unluckily",
"lump",
"lumps",
"lunch",
"lunches",
"lung",
"lungs",
"mad",
"madness",
"madly",
"maddest",
"madder",
"madman",
"mail",
"mails",
"mailing",
"mailed",
"male",
"males",
"manage",
"managing",
"manages",
"managers",
"manager",
"managed",
"management",
"map",
"maps",
"mat",
"mats",
"match",
"matching",
"matches",
"matched",
"meal",
"meals",
"meantime",
"meanwhile",
"meat",
"meats",
"mechanic",
"mechanics",
"mechanical",
"mechanically",
"medicine",
"medicines",
"medic",
"melt",
"melts",
"melting",
"melted",
"mend",
"mends",
"mending",
"mended",
"merchant",
"merchants",
"mercy",
"merciful",
"mercies",
"merry",
"merriness",
"merriest",
"merrier",
"merrily",
"message",
"messages",
"messenger",
"messengers",
"metre",
"metres",
"mild",
"mildness",
"mildly",
"mildest",
"milder",
"mill",
"mills",
"milling",
"miller",
"milled",
"milligram",
"milligrams",
"millilitre",
"millilitres",
"millimetre",
"millimetres",
"mineral",
"minerals",
"miserable",
"miserably",
"mistake",
"mistaking",
"mistakes",
"mistakenly",
"mistaken",
"unmistakably",
"unmistakable",
"mix",
"mixtures",
"mixture",
"mixing",
"mixes",
"mixed",
"model",
"models",
"modelling",
"modelled",
"modeling",
"modeled",
"moderate",
"moderately",
"moderation",
"modest",
"modesty",
"modestly",
"monkey",
"monkeys",
"motion",
"motions",
"motioning",
"motioned",
"motionless",
"mouse",
"mice",
"mud",
"muddy",
"multiply",
"multiplying",
"multiplies",
"multiplied",
"multiple",
"multiplication",
"murder",
"murders",
"murdering",
"murderers",
"murderer",
"murdered",
"mystery",
"mysteriously",
"mysterious",
"mysteries",
"nail",
"nails",
"nailing",
"nailed",
"narrow",
"narrowed",
"narrower",
"narrowest",
"narrowing",
"narrowly",
"narrowness",
"narrows",
"neat",
"neatness",
"neatly",
"neatest",
"neater",
"neck",
"necks",
"needle",
"needling",
"needles",
"needled",
"neglect",
"neglects",
"neglecting",
"neglected",
"nephew",
"nephews",
"nest",
"nests",
"nesting",
"nested",
"net",
"nets",
"nice",
"nicest",
"nicer",
"niceness",
"nicely",
"niece",
"nieces",
"noise",
"noisy",
"noisily",
"noises",
"nonsense",
"nonsensical",
"noon",
"nose",
"noses",
"noun",
"nouns",
"nuisance",
"nuisances",
"nurse",
"nursing",
"nurses",
"nursed",
"nut",
"nuts",
"oar",
"oars",
"obey",
"obeys",
"obeying",
"obeyed",
"obedience",
"obediently",
"obedient",
"disobey",
"disobeyed",
"disobeying",
"ocean",
"oceans",
"offend",
"offenses",
"offense",
"offends",
"offending",
"offended",
"offensive",
"offensively",
"inoffensive",
"inoffensively",
"omit",
"omitting",
"omitted",
"omits",
"omission",
"omissions",
"onwards",
"onward",
"oppose",
"opposing",
"opposes",
"opposed",
"opposite",
"opposites",
"orange",
"oranges",
"organ",
"organs",
"origin",
"origins",
"originates",
"originated",
"original",
"originally",
"originality",
"ornament",
"ornaments",
"ornamental",
"outline",
"outlined",
"outlines",
"outlining",
"overcome",
"overcoming",
"overcomes",
"overcame",
"pack",
"packs",
"packing",
"packed",
"package",
"packages",
"pad",
"pads",
"padding",
"padded",
"pain",
"pains",
"painful",
"painfully",
"pair",
"pairs",
"pairing",
"paired",
"pale",
"paling",
"pales",
"paled",
"paleness",
"pan",
"pans",
"parcel",
"parcels",
"parcelled",
"pardon",
"pardons",
"pardoning",
"pardoned",
"unpardonable",
"parent",
"parents",
"parental",
"parentage",
"park",
"parks",
"parking",
"parked",
"permanent",
"permanently",
"passage",
"passages",
"passenger",
"passengers",
"paste",
"pasting",
"pastes",
"pasted",
"path",
"paths",
"patient",
"patiently",
"patience",
"patriotic",
"pattern",
"patterns",
"patterned",
"pause",
"pausing",
"pauses",
"paused",
"paw",
"paws",
"pearl",
"pearls",
"peculiar",
"peculiarly",
"peculiarity",
"peculiarities",
"pen",
"pens",
"pencil",
"pencils",
"penny",
"pennies",
"perfect",
"perfected",
"perfecting",
"perfectly",
"perfects",
"perfection",
"perform",
"performs",
"performing",
"performers",
"performed",
"performance",
"performances",
"persuade",
"persuasion",
"persuades",
"persuaded",
"persuading",
"persuasive",
"persuasively",
"pet",
"pets",
"photograph",
"photography",
"photographs",
"photographing",
"photographers",
"photographer",
"photographed",
"pick",
"picks",
"picking",
"pickers",
"picker",
"picked",
"pig",
"pigs",
"pigeon",
"pigeons",
"pile",
"piling",
"piles",
"piled",
"pin",
"pins",
"pinned",
"pinning",
"pinch",
"pinching",
"pinches",
"pinched",
"pink",
"pinkness",
"pinkest",
"pinker",
"pint",
"pints",
"pipe",
"pipes",
"pity",
"pitying",
"pities",
"pitied",
"plane",
"planes",
"plaster",
"plasters",
"plastering",
"plastered",
"plate",
"plates",
"plenty",
"plentiful",
"plough",
"ploughs",
"ploughing",
"ploughed",
"plural",
"plurals",
"pocket",
"pockets",
"poem",
"poetry",
"poetic",
"poet",
"poems",
"poison",
"poisons",
"poisonous",
"poisoning",
"poisoned",
"police",
"policing",
"policewomen",
"polices",
"policemen",
"policeman",
"policed",
"polish",
"polishing",
"polishes",
"polished",
"polite",
"politest",
"politer",
"politeness",
"pool",
"pools",
"pooling",
"postpone",
"postponing",
"postpones",
"postponed",
"pot",
"potting",
"potted",
"pots",
"pour",
"pours",
"pouring",
"poured",
"powder",
"powders",
"powdery",
"practical",
"practically",
"practise",
"practises",
"practised",
"practising",
"practice",
"practices",
"praise",
"praising",
"praises",
"praised",
"pray",
"prays",
"praying",
"prayers",
"prayer",
"prayed",
"preach",
"preaching",
"preaches",
"preachers",
"preacher",
"preached",
"precious",
"prefer",
"prefers",
"preferring",
"preferred",
"preferable",
"preference",
"preferences",
"preferential",
"prejudice",
"prejudices",
"prejudiced",
"unprejudiced",
"preserve",
"preserved",
"preserves",
"preserving",
"preservation",
"pretend",
"pretends",
"pretending",
"pretended",
"pretence",
"pride",
"prides",
"priest",
"priests",
"print",
"printed",
"printers",
"printing",
"prints",
"reprint",
"reprints",
"reprinting",
"reprinted",
"prison",
"prisons",
"prisoners",
"prisoner",
"imprison",
"prize",
"prized",
"prizes",
"probable",
"probably",
"probability",
"probabilities",
"improbable",
"procession",
"processions",
"profession",
"professions",
"programme",
"program",
"programed",
"programing",
"programmed",
"programmes",
"programming",
"programs",
"prompt",
"prompts",
"promptly",
"prompting",
"prompted",
"pronounce",
"pronouncing",
"pronounces",
"pronounced",
"proud",
"proudly",
"proudest",
"prouder",
"pump",
"pumps",
"pumping",
"pumped",
"punctual",
"punctually",
"punctuality",
"punish",
"punishing",
"punishes",
"punished",
"punishment",
"punishments",
"pupil",
"pupils",
"pure",
"purest",
"purer",
"pureness",
"purely",
"impure",
"impurity",
"purple",
"purplish",
"push",
"pushing",
"pushes",
"pushed",
"puzzle",
"puzzling",
"puzzles",
"puzzled",
"qualify",
"qualifying",
"qualifies",
"qualified",
"qualification",
"qualifications",
"quarrel",
"quarrels",
"quarrelling",
"quarrelled",
"quarreling",
"quarreled",
"quart",
"quarts",
"quick",
"quickness",
"quickly",
"quickest",
"quicker",
"quiet",
"quieter",
"quietest",
"quietly",
"quietness",
"rabbit",
"rabbits",
"radio",
"radios",
"radioed",
"rail",
"railways",
"railway",
"rails",
"railroads",
"railroad",
"rain",
"rains",
"raining",
"rained",
"raindrops",
"raindrop",
"rake",
"raking",
"rakes",
"raked",
"rapid",
"rapidly",
"rapidity",
"rare",
"rarest",
"rarer",
"rareness",
"rarely",
"rat",
"rats",
"raw",
"rawness",
"rawest",
"rawer",
"ray",
"rays",
"razor",
"razors",
"recommend",
"recommends",
"recommending",
"recommended",
"recommendation",
"recommendations",
"refer",
"refers",
"referring",
"referred",
"reflect",
"reflects",
"reflecting",
"reflected",
"reflective",
"reflection",
"reflections",
"refresh",
"refreshes",
"refreshing",
"refreshed",
"refreshment",
"regular",
"irregular",
"regularly",
"irregularly",
"regret",
"regretting",
"regretful",
"regretted",
"regrets",
"rejoice",
"rejoicing",
"rejoices",
"rejoiced",
"relieve",
"relieving",
"relieves",
"relieved",
"relief",
"remedy",
"remedying",
"remedies",
"remedied",
"remind",
"reminds",
"reminding",
"reminded",
"reminder",
"rent",
"renting",
"rented",
"rents",
"repair",
"repairs",
"repairing",
"repaired",
"repeat",
"repeated",
"repeatedly",
"repeating",
"repeats",
"replace",
"replacing",
"replaces",
"replaced",
"replacement",
"replacements",
"replaceable",
"irreplaceable",
"reproduce",
"reproduces",
"reproducing",
"reproduced",
"reproduction",
"reproductions",
"reputation",
"reputations",
"request",
"requests",
"requesting",
"requested",
"rescue",
"rescuing",
"rescues",
"rescued",
"resign",
"resigns",
"resigning",
"resigned",
"resignation",
"resignations",
"resist",
"resists",
"resisting",
"resisted",
"resistible",
"resistance",
"responsible",
"irresponsible",
"responsibility",
"responsibilities",
"restaurant",
"restaurants",
"retire",
"retiring",
"retires",
"retired",
"retirement",
"revenge",
"revenges",
"review",
"reviews",
"reviewing",
"reviewed",
"reward",
"rewards",
"rewarding",
"rewarded",
"ribbon",
"ribbons",
"rice",
"rid",
"rids",
"ridding",
"ripe",
"ripest",
"riper",
"ripeness",
"risk",
"risks",
"risking",
"risked",
"risky",
"rival",
"rivals",
"rivalry",
"rivalries",
"rivalling",
"rivalled",
"roar",
"roars",
"roaring",
"roared",
"roast",
"roasts",
"roasting",
"roasted",
"rob",
"robs",
"robbing",
"robber",
"robbed",
"robbers",
"rod",
"rods",
"roof",
"roofs",
"root",
"roots",
"rooted",
"rope",
"ropes",
"rot",
"rotting",
"rotted",
"rots",
"rotten",
"row",
"rows",
"rowing",
"rowed",
"rub",
"rubs",
"rubbing",
"rubbed",
"rubber",
"rubbers",
"rubbish",
"rude",
"rudest",
"ruder",
"rudeness",
"rudely",
"rug",
"rugs",
"ruin",
"ruined",
"ruining",
"ruins",
"rush",
"rushed",
"rushes",
"rushing",
"rust",
"rusty",
"rusts",
"rusting",
"rusted",
"sacred",
"sacredness",
"sacrifice",
"sacrificing",
"sacrifices",
"sacrificed",
"sacrificial",
"sad",
"sadness",
"sadly",
"saddest",
"sadder",
"sadden",
"saddle",
"saddling",
"saddles",
"saddled",
"sake",
"sakes",
"salary",
"salaries",
"sample",
"sampling",
"samples",
"sampled",
"sand",
"sandy",
"sands",
"satisfy",
"satisfying",
"satisfies",
"satisfied",
"satisfactory",
"satisfaction",
"dissatisfaction",
"dissatisfying",
"dissatisfy",
"dissatisfies",
"dissatisfied",
"sauce",
"sauces",
"saucer",
"saucers",
"saws",
"sawing",
"sawed",
"scale",
"scales",
"scatter",
"scatters",
"scattering",
"scattered",
"scent",
"scents",
"scenting",
"scented",
"scissors",
"scold",
"scolds",
"scolding",
"scolded",
"scorn",
"scorns",
"scorning",
"scorned",
"scornful",
"scornfully",
"scrape",
"scraping",
"scrapes",
"scraped",
"scratch",
"scratching",
"scratches",
"scratched",
"screen",
"screens",
"screening",
"screened",
"screw",
"screws",
"screwing",
"screwed",
"search",
"searching",
"searches",
"searched",
"seed",
"seeds",
"seeding",
"seeded",
"seedling",
"seize",
"seized",
"seizes",
"seizing",
"seldom",
"self",
"selfishness",
"selfishly",
"selfish",
"selves",
"sentence",
"sentences",
"severe",
"severity",
"severest",
"severer",
"severeness",
"severely",
"sew",
"sews",
"sewn",
"sewing",
"sewed",
"shade",
"shadings",
"shading",
"shades",
"shaded",
"shallow",
"shallowness",
"shallowest",
"shallower",
"shame",
"shaming",
"shames",
"shameful",
"shamefully",
"shamed",
"sharp",
"sharpness",
"sharply",
"sharpest",
"sharper",
"sharpens",
"sharpening",
"sharpened",
"sharpen",
"shave",
"shaved",
"shaving",
"shaves",
"sheep",
"sheet",
"sheets",
"shelf",
"shelves",
"shell",
"shells",
"shelling",
"shelled",
"shelter",
"shelters",
"sheltering",
"sheltered",
"shield",
"shields",
"shielding",
"shielded",
"shilling",
"shillings",
"shirt",
"shirts",
"shock",
"shocks",
"shocking",
"shocked",
"shoe",
"shoeing",
"shoes",
"shop",
"shops",
"shopping",
"shoppers",
"shopper",
"shopped",
"shout",
"shouted",
"shouting",
"shouts",
"shower",
"showers",
"showering",
"showered",
"shut",
"shutting",
"shuts",
"sick",
"sickness",
"sickly",
"sickest",
"sicker",
"signal",
"signals",
"signalling",
"signalled",
"silk",
"silks",
"sincere",
"sincerely",
"sincerity",
"sink",
"sunk",
"sinks",
"sinking",
"sank",
"skill",
"skilled",
"skills",
"skilfully",
"skin",
"skins",
"skinning",
"skinned",
"skirt",
"skirts",
"slave",
"slaves",
"slavery",
"slide",
"sliding",
"slides",
"slid",
"slight",
"slighted",
"slighter",
"slightest",
"slighting",
"slightly",
"slights",
"slow",
"slowed",
"slower",
"slowest",
"slowing",
"slowly",
"slows",
"slip",
"slips",
"slipping",
"slipped",
"slope",
"sloping",
"slopes",
"sloped",
"smell",
"smelt",
"smells",
"smelling",
"smelled",
"smoke",
"smoking",
"smokes",
"smokers",
"smokeless",
"smoked",
"smooth",
"smooths",
"smoothly",
"smoothing",
"smoothed",
"snake",
"snakes",
"soap",
"soaps",
"sock",
"socks",
"soil",
"soiled",
"soiling",
"soils",
"solemn",
"solemnly",
"solid",
"solidify",
"solidly",
"solids",
"solve",
"solving",
"solves",
"solved",
"solution",
"solutions",
"sore",
"sores",
"sorry",
"sorriness",
"sorriest",
"sorrier",
"soup",
"soups",
"sour",
"sours",
"sourly",
"souring",
"soured",
"sow",
"sows",
"sowing",
"sowed",
"spade",
"spades",
"spare",
"sparing",
"spares",
"spared",
"spell",
"spelt",
"spells",
"spelling",
"spelled",
"spill",
"spills",
"spilling",
"spilled",
"spin",
"spun",
"spins",
"spinning",
"spit",
"spits",
"spitting",
"spat",
"splendid",
"splendidly",
"split",
"splitting",
"splits",
"spoil",
"spoils",
"spoiling",
"spoiled",
"spoon",
"spoons",
"spoonful",
"sport",
"sporting",
"sports",
"staff",
"staffs",
"stain",
"stains",
"staining",
"stained",
"stairs",
"upstairs",
"downstairs",
"stamp",
"stamps",
"stamping",
"stamped",
"steady",
"unsteady",
"unsteadily",
"steadying",
"steadily",
"steadies",
"steadied",
"steal",
"stole",
"stolen",
"steals",
"stealing",
"steam",
"steams",
"steaming",
"steamed",
"steep",
"steepness",
"steepest",
"steeper",
"steer",
"steers",
"steering",
"steered",
"stem",
"stems",
"stemmed",
"stick",
"sticking",
"sticks",
"stuck",
"stiff",
"stiffness",
"stiffest",
"stiffer",
"stiffens",
"stiffening",
"stiffened",
"stiffen",
"sting",
"stung",
"stings",
"stinging",
"stir",
"stirs",
"stirring",
"stirred",
"stocking",
"stockings",
"stomach",
"stomachs",
"storm",
"stormed",
"storming",
"storms",
"stove",
"stoves",
"straight",
"straightened",
"straightening",
"straightens",
"straighter",
"straightest",
"strap",
"straps",
"strapping",
"strapped",
"straw",
"straws",
"stretch",
"stretched",
"stretches",
"stretching",
"strict",
"strictness",
"strictly",
"strictest",
"stricter",
"string",
"strings",
"stringing",
"stringed",
"strip",
"strips",
"stripped",
"stripping",
"stripe",
"stripes",
"striped",
"striping",
"stuff",
"stuffs",
"stuffing",
"stuffed",
"stupid",
"stupidity",
"stupidly",
"stupidest",
"stupider",
"suck",
"sucks",
"sucking",
"sucked",
"sudden",
"suddenly",
"suddenness",
"sugar",
"sugars",
"sugary",
"suit",
"suits",
"suiting",
"suited",
"suitability",
"suitable",
"unsuitable",
"supper",
"suppers",
"suspect",
"suspects",
"suspecting",
"suspected",
"unsuspecting",
"suspicion",
"suspicions",
"suspicious",
"suspiciously",
"swallow",
"swallows",
"swallowing",
"swallowed",
"swear",
"sworn",
"swore",
"swears",
"swearing",
"sweat",
"sweats",
"sweating",
"sweep",
"swept",
"sweeps",
"sweeping",
"swell",
"swollen",
"swells",
"swelling",
"swelled",
"swim",
"swum",
"swims",
"swimming",
"swam",
"swing",
"swung",
"swings",
"swinging",
"sympathy",
"sympathies",
"sympathetic",
"sympathetically",
"tail",
"tails",
"tailor",
"tailors",
"tailoring",
"tailored",
"tall",
"tallest",
"taller",
"tame",
"taming",
"tames",
"tamed",
"tap",
"taps",
"tapping",
"tapped",
"taste",
"tasted",
"tastes",
"tasting",
"taxi",
"taxis",
"tea",
"teas",
"telegraph",
"telegraphs",
"telegraphing",
"telegraphed",
"telephone",
"telephoning",
"telephones",
"telephoned",
"phones",
"phoning",
"phoned",
"phone",
"temper",
"tempers",
"temperature",
"temperatures",
"tempt",
"tempts",
"tempting",
"tempted",
"tend",
"tends",
"tended",
"tendency",
"tendencies",
"tender",
"tenderness",
"tenderly",
"tent",
"tents",
"terrible",
"terribly",
"thank",
"thanks",
"thanking",
"thankfully",
"thankful",
"thanked",
"theatre",
"theatres",
"theatrical",
"thick",
"thickness",
"thickly",
"thickest",
"thicker",
"thief",
"thieves",
"thin",
"thins",
"thinning",
"thinnest",
"thinness",
"thinner",
"thinned",
"thinly",
"thirst",
"thirsts",
"thorn",
"thorns",
"thorough",
"thoroughness",
"thoroughly",
"thread",
"threads",
"threading",
"threaded",
"threat",
"threats",
"threatens",
"threatening",
"threatened",
"threaten",
"throat",
"throats",
"thumb",
"thumbs",
"thunder",
"thunders",
"thundering",
"thundered",
"ticket",
"tickets",
"tide",
"tides",
"tidal",
"tidy",
"tidying",
"tidies",
"tidied",
"tie",
"untied",
"untie",
"tying",
"ties",
"tied",
"tight",
"tightness",
"tightest",
"tighter",
"tightly",
"tightens",
"tightening",
"tightened",
"tighten",
"tin",
"tins",
"tip",
"tips",
"tipping",
"tipped",
"tire",
"tiring",
"tires",
"tired",
"title",
"titled",
"titles",
"tobacco",
"tobaccos",
"toe",
"toes",
"tomorrow",
"tongue",
"tongues",
"tonight",
"tool",
"tools",
"tooth",
"teeth",
"toothless",
"tough",
"toughness",
"toughest",
"tougher",
"tour",
"tours",
"touring",
"toured",
"tourist",
"tourists",
"towel",
"towels",
"tower",
"towers",
"towering",
"toy",
"toys",
"track",
"tracks",
"tracking",
"tracked",
"translate",
"translating",
"translates",
"translated",
"translation",
"translations",
"trap",
"traps",
"trapping",
"trapped",
"tray",
"trays",
"treasure",
"treasuring",
"treasures",
"treasured",
"treasury",
"treasurer",
"treat",
"treats",
"treating",
"treated",
"treatment",
"treatments",
"tremble",
"trembling",
"trembles",
"trembled",
"tribe",
"tribes",
"trick",
"trickster",
"tricks",
"tricking",
"tricked",
"trip",
"trips",
"tripping",
"tripped",
"trunk",
"trunks",
"tube",
"tubes",
"tune",
"tuning",
"tunes",
"tuned",
"twist",
"twists",
"twisting",
"twisted",
"typical",
"typically",
"ugly",
"ugliness",
"ugliest",
"uglier",
"umbrella",
"umbrellas",
"uncle",
"uncles",
"universe",
"universal",
"upper",
"upright",
"upset",
"upsetting",
"upsets",
"upwards",
"upward",
"urge",
"urging",
"urges",
"urged",
"vain",
"vainly",
"vainest",
"vainer",
"veil",
"veils",
"veiling",
"veiled",
"verb",
"verbs",
"verse",
"verses",
"violent",
"violently",
"violence",
"nonviolent",
"vowel",
"vowels",
"voyage",
"voyaging",
"voyages",
"voyaged",
"waist",
"waists",
"wake",
"woke",
"waking",
"wakes",
"waken",
"woken",
"wander",
"wanders",
"wandering",
"wandered",
"warm",
"warms",
"warmly",
"warming",
"warmed",
"warmth",
"warn",
"warned",
"warning",
"warns",
"wash",
"washed",
"washes",
"washing",
"waste",
"wasted",
"wastes",
"wasting",
"wasteful",
"wastefully",
"wax",
"waxing",
"waxes",
"waxed",
"weak",
"weakly",
"weakness",
"weaken",
"weapon",
"weapons",
"weather",
"weathers",
"weathering",
"weathered",
"weave",
"wove",
"weaving",
"weaves",
"weaved",
"weed",
"weeds",
"weeding",
"weeded",
"weigh",
"weights",
"weight",
"weighs",
"weighing",
"weighed",
"wet",
"wetting",
"wets",
"wheat",
"wheel",
"wheels",
"wheeler",
"wheeled",
"whip",
"whips",
"whipping",
"whipped",
"whisper",
"whispers",
"whispering",
"whispered",
"whistle",
"whistling",
"whistles",
"whistled",
"wicked",
"wickedness",
"wickedest",
"wickeder",
"wickedly",
"widow",
"widows",
"widower",
"widowed",
"wine",
"wining",
"wines",
"wined",
"wing",
"wings",
"winging",
"winged",
"wipe",
"wiping",
"wipes",
"wiped",
"wiper",
"wipers",
"wire",
"wiring",
"wires",
"wired",
"witness",
"witnessing",
"witnesses",
"witnessed",
"wool",
"wools",
"woollen",
"woolly",
"worm",
"worms",
"worming",
"wormed",
"worry",
"worrying",
"worries",
"worried",
"worse",
"worst",
"worsen",
"worship",
"worshipping",
"worships",
"worshipped",
"wrap",
"wrapping",
"wraps",
"wrapper",
"wrappers",
"wrapped",
"wreck",
"wrecked",
"wrecking",
"wrecks",
"wrist",
"wrists",
"yard",
"yards",
"yellow",
"yellowish",
"zero",
"zeros",
"advisors",
"aunty",
"broadcasted",
"bussing",
"cappings",
"cruellest",
"crueller",
"kneeled",
"orangish",
"ridded",
"shoed",
"skilful",
"misspelt",
"sweated",
"wheats",
}

is_word_gsl2 = lambda w : w in word_gsl2

def submit(port, host='127.0.0.1',  db=0):
	import redis
	r = redis.Redis(host=host, port=port, db=db)
	for w in word_gsl2: 
		r.sadd("dict:gsl2", w )
		r.hset('dict:word_level', w, 'gsl2')
	print(">> finished loading gsl2")

if __name__ == '__main__':
	submit(6311)


# 2022.3.2  /home/ubuntu/.local/lib/python3.8/site-packages/uvirun/sqlite.py
import json,fire,os,sqlite3
from uvirun import * 

@app.get("/sqlite/dbnames")
def sqlite_dbnames():
	''' name list of the given folder '''
	return [k for k,v in fire.conns.items()]

@app.get("/sqlite")
def sqlite_query(sql:str="select * from ssi limit 2"):
	''' single file mode '''
	return [row for row in fire.conn.execute(sql).fetchall()]

query = lambda name, sql:  [row for row in fire.conns[name].execute(sql).fetchall()]
@app.post("/sqlite/query")
def sqlite_query(names:list=["gzjc","clec"], sql:str="select * from ssi limit 2"):
	''' query from multiple dbs '''
	return { name : query(name, sql) for name in names }

class util(object):
	def __init__(self): pass 

	def single(self, dbfile, port) :
		''' open a single sqlite db, ie:  python -m uvirun.sqlite test.sqlite 13306 '''
		fire.conn = sqlite3.connect(dbfile, check_same_thread=False)  
		fire.conn.execute('PRAGMA synchronous=OFF')
		uvicorn.run(app, host='0.0.0.0', port=port)

	def folder(self, folder, port, suffix=".sqlite") :
		''' open a folder of sqlite db, ie: python -m uviru.sqlite corpus 13306 '''
		fire.conns = {}
		for root, dirs, files in os.walk(folder):
			for file in files: 
				if file.endswith(suffix):	
					try: 
						print ("root:|", root, "\tfile:|", file, "\tfolder:|", folder, flush=True)
						fire.conns[ file.split('.')[0] ] = sqlite3.connect(f"{root}/{file}", check_same_thread=False)  
					except Exception as e:
						print ("ex:", e, "\t|", file)
		uvicorn.run(app, host='0.0.0.0', port=port)

if __name__ == "__main__":  
	fire.Fire(util)


#pip install -U sentence-transformers
from sentence_transformers import SentenceTransformer
model = SentenceTransformer('all-MiniLM-L6-v2')

#Our sentences we like to encode
sentences = ['This framework generates embeddings for each input sentence',
    'Sentences are passed as a list of string.',
    'The quick brown fox jumps over the lazy dog.']

#Sentences are encoded by calling model.encode()
embeddings = model.encode(sentences)

#Print the embeddings
for sentence, embedding in zip(sentences, embeddings):
    print("Sentence:", sentence)
    print("Embedding:", embedding)
    print("")


# 2022.3.5
from app.dmdsk import *

def run():
	
	st.title("rid-scores")
	rid = st.text_input("input a rid", "2575450")
	res = {eidv:redis.dsk.hget(eidv,'score') for eidv in eidv_list(int(rid))}
	st.write(res) 

if __name__ == '__main__': run()


# 2022.2.18
import json, sys, time, fire, os, spacy

if not hasattr(spacy, 'snts'): 
	from spacy.lang import en
	spacy.sntbr		= (inst := en.English(), inst.add_pipe("sentencizer"))[0]
	spacy.snts		= lambda essay: [ snt.text.strip() for snt in  spacy.sntbr(essay).sents]
	spacy.sntpid	= lambda essay: (pid:=0, [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text.strip(), pid))[-1] for snt in  spacy.sntbr(essay).sents] )[-1]

def postag_nlp():
	nlp = spacy.load('en_core_web_sm')  # 3.1.1
	#t = nlp.get_pipe('tagger')
	#>>> nlp.pipe_names #['tok2vec', 'tagger', 'parser', 'attribute_ruler', 'lemmatizer', 'ner']
	#[nlp.remove_pipe(n) for n in ['tok2vec','parser', 'attribute_ruler','lemmatizer', 'ner'] ]
	[nlp.remove_pipe(n) for n in ['parser',  'ner'] ]
	return nlp 

def readline(infile, sepa=None):
	with open(infile, 'r') as fp:
		while True:
			line = fp.readline()
			if not line: break
			yield line.strip().split(sepa) if sepa else line.strip()

class util(object):
	def __init__(self): pass 

	def down(self, ibeg,iend):
		for i in range(ibeg, iend):  
			j = str(10000 + i)
			os.system(f"wget https://huggingface.co/datasets/allenai/c4/resolve/main/en/c4-train.0{j[1:]}-of-01024.json.gz")

	def compress(self):
		''' 0..1023 '''
		for i in range(1024):  # 0..1023
			j = str(10000 + i)
			os.system(f"gzip c4-train.0{j[1:]}-of-01024.json")

	def sntbr(self, infile, outfile=None):
		'''  'text' -> 'snts' 2022.2.19	'''
		print("started:", infile , flush=True)
		if not outfile: outfile = infile + ".snts"
		with open(outfile, 'w') as fw : 
			for line in readline(infile): 
				try:
					arr =json.loads(line) 
					arr['snts'] = spacy.snts(arr.get('text','').strip())
					del arr['text']
					fw.write( json.dumps(arr) + "\n")
				except Exception as e:
					print("ex:", e, line)
		print("finished:", infile , flush=True)

	def sntbr_all(self):
		''' 0..1023 '''
		for i in range(1024):  # 0..1023
			j = str(10000 + i)
			filename = f"c4-train.0{j[1:]}-of-01024.json"
			print( i,  filename, flush=True)
			self.sntbr(filename) 

	def postag(self, infile, outfile=None):
		''' c0.snts0 => c0.snts.postag '''
		if not outfile: outfile = infile + ".postag"
		if os.path.exists(outfile): 
			print ("already exists:", outfile, flush=True)
			return 
		nlp = postag_nlp()
		with open(outfile, 'w') as fw:
			for line in readline(infile):
				try:
					arr = json.loads(line.strip())
					for snt in arr.get('snts',[]):
						doc = nlp(snt)
						poslist = [(t.text, t.pos_, t.tag_) for t in doc]
						fw.write(json.dumps(poslist) + "\n") # prepared for n-gram counting 
				except Exception as e:
					print("ex:", e, line)
		print ("finished:", infile, outfile)

if __name__ == '__main__':
	fire.Fire(util)

'''
ubuntu@dsk-162-10-ubuntu:/ftp/c4data$ nohup python c4data.py sntbr_all &
[1] 45656

'''


# 2020-9-10 #https://github.com/chrisjbryant/errant
import errant, spacy		
errant.annotator = errant.load('en', nlp=spacy.load('en_core_web_sm'))

errant_filter = {"R:SPELL":"t.tag_ not in ('NNP','NNPS')"} #R:SPELL=NNP -> t.tag_ not in ('NNP','NNPS') |   A and B , in one unit
errant_type = {
"M:DET":"{ 'error': '冠词缺失', 'explanation':f'建议添加冠词<b>{edit.c_str}</b>'}",
"R:ADV":"{ 'error': '副词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"M:NOUN:POSS":"{ 'error': '名词所有格缺失', 'explanation':f'建议添加名词所有格<b>{edit.c_str}</b>'}",
"M:PREP":"{ 'error': '介词缺失', 'explanation':f'建议添加介词<b>{edit.c_str}</b>'}",
"R:PRON":"{ 'error': '代词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"UNK":"{ 'error': '未知错误', 'explanation':f'检测到但未修正错误'}",
"R:VERB:SVA":"{ 'error': '主谓不一致', 'explanation':f'请检查动词<b>{edit.o_str}</b>形态'}",
"R:ADJ":"{ 'error': '形容词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"M:CONTR":"{ 'error': '缩略形式缺失', 'explanation':f'建议添加缩略形式<b>{edit.c_str}</b>'}",
"R:ADJ:FORM":"{ 'error': '形容词形式错误', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"M:VERB:TENSE":"{ 'error': '动词时态缺失', 'explanation':f'建议添加动词时态<b>{edit.c_str}</b>'}",
"R:DET":"{ 'error': '冠词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"U:VERB:TENSE":"{ 'error': '动词时态多余', 'explanation':f'动词时态<b>{edit.o_str}</b>疑似多余'}",
"R:NOUN":"{ 'error': '名词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:NOUN:POSS":"{ 'error': '名词所有格误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:VERB:TENSE":"{ 'error': '动词时态误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"U:VERB:FORM":"{ 'error': '动词形式多余', 'explanation':f'动词形式<b>{edit.o_str}</b>疑似多余'}",
"U:PUNCT":"{ 'error': '标点符号多余', 'explanation':f'标点符号<b>{edit.o_str}</b>疑似多余'}",
"U:NOUN":"{ 'error': '名词多余', 'explanation':f'名词<b>{edit.o_str}</b>疑似多余'}",
"R:ORTH":"{ 'error': '大小写/或空格错误', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:NOUN:INFL":"{ 'error': '名词词形变化错误', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:NOUN:NUM":"{ 'error': '名词单复数错误', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"M:CONJ":"{ 'error': '连词缺失', 'explanation':f'建议添加连词<b>{edit.c_str}</b>'}",
"R:VERB":"{ 'error': '动词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"M:PRON":"{ 'error': '代词缺失', 'explanation':f'建议添加代词<b>{edit.c_str}</b>'}",
"M:NOUN":"{ 'error': '名词缺失', 'explanation':f'建议添加名词<b>{edit.c_str}</b>'}",
"M:ADV":"{ 'error': '副词缺失', 'explanation':f'建议添加副词<b>{edit.c_str}</b>'}",
"U:CONJ":"{ 'error': '连词多余', 'explanation':f'连词<b>{edit.o_str}</b>疑似多余'}",
"R:PUNCT":"{ 'error': '标点符号误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"U:ADV":"{ 'error': '副词多余', 'explanation':f'副词<b>{edit.o_str}</b>疑似多余'}",
"R:CONTR":"{ 'error': '缩略形式错误', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"U:PREP":"{ 'error': '介词多余', 'explanation':f'介词<b>{edit.o_str}</b>疑似多余'}",
"U:DET":"{ 'error': '冠词多余', 'explanation':f'冠词<b>{edit.o_str}</b>疑似多余'}",
"M:VERB:FORM":"{ 'error': '动词形式缺失', 'explanation':f'建议添加动词形式<b>{edit.c_str}</b>'}",
"M:ADJ":"{ 'error': '形容词缺失', 'explanation':f'建议添加形容词<b>{edit.c_str}</b>'}",
"M:VERB":"{ 'error': '动词缺失', 'explanation':f'建议添加动词<b>{edit.c_str}</b>'}",
"U:CONTR":"{ 'error': '缩略形式多余', 'explanation':f'缩略形式<b>{edit.o_str}</b>疑似多余'}",
"U:VERB":"{ 'error': '动词多余', 'explanation':f'动词<b>{edit.o_str}</b>疑似多余'}",
"U:PART":"{ 'error': '与动词构成短语动词的副词或介词多余', 'explanation':f'与动词构成短语动词的副词或介词<b>{edit.o_str}</b>疑似多余'}",
"R:OTHER":"{ 'error': '单词误用', 'explanation':f'建议修改为<b>{edit.c_str}</b>'}",
"U:NOUN:POSS":"{ 'error': '名词所有格多余', 'explanation':f'名词所有格<b>{edit.o_str}</b>疑似多余'}",
"R:WO":"{ 'error': '语序错误', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:PREP":"{ 'error': '介词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:VERB:INFL":"{ 'error': '动词词形变化错误', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"U:PRON":"{ 'error': '代词多余', 'explanation':f'代词<b>{edit.o_str}</b>疑似多余'}",
"R:CONJ":"{ 'error': '连词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:SPELL":"{ 'error': '拼写错误', 'explanation':f'请检查<b>{edit.o_str}</b>拼写'}",
"R:PART":"{ 'error': '与动词构成短语动词的副词或介词误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"U:SPACE":"{ 'error': '空格多余', 'explanation':f'建议删除空格'}",
"U:OTHER":"{ 'error': '单词冗余', 'explanation':f'建议删除<b>{edit.o_str} </b>'}",
"R:MORPH":"{ 'error': '词性误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"R:VERB:FORM":"{ 'error': '动词形式误用', 'explanation':f'建议<b>{edit.o_str}</b>修改为<b>{edit.c_str}</b>'}",
"M:OTHER":"{ 'error': '单词缺失', 'explanation':f'建议添加<b>{edit.c_str}</b>'}",
"U:ADJ":"{ 'error': '形容词多余', 'explanation':f'形容词<b>{edit.o_str}</b>疑似多余'}",
"M:PART":"{ 'error': '与动词构成短语动词的副词或介词缺失', 'explanation':f'建议添加与动词构成短语动词的副词或介词<b>{edit.c_str}</b>'}",
"M:PUNCT":"{ 'error': '标点符号缺失', 'explanation':f'建议添加标点符号<b>{edit.c_str}</b>'}",
}

def feed_errant(r): 
	errant_filter.update( r.hgetall('errant:filter'))
	errant_type.update( r.hgetall('errant:type'))

def reload_errant(r):
	[errant_type.update({k:v}) for k,v in r.hgetall('errant:type').items()]
	for k,code in r.hgetall("errant:filter").items(): #R:SPELL -> t.tag_ not in ('NNP','NNPS')
		errant_filter.update({k:code})
	return r.hlen('errant:type')

def from_edit(doc, edit):
	op = 's' if edit.type.startswith("R:") else 'i' if edit.type.startswith("M:") else 'd' if edit.type.startswith("U:") else 'x' #missed, unnecessay
	hit =  {'op': op, 'position': doc[edit.o_start].idx, 'ibeg':edit.o_start,'text':edit.o_str, 'replaceText':edit.c_str, 'cate': edit.type, 'error': edit.type, 'explanation':f'Please check <b>{edit.o_str}</b>', 'type':'grammar'}
	try:
		cate_msg = errant_type.get(edit.type,'{}')  # cached_hget('__errant:type', edit.type) 
		hit.update( eval( cate_msg, {'doc':doc, 'hit':hit, 'edit':edit})   )
	except Exception as e:
		print("from_edit ex:", e, hit)
	return hit

def pass_filter(doc, edit): # R:SPELL=NNP -> t.tag_ not in ('NNP','NNPS')
	try:
		code =  errant_filter.get(edit.type, "True" ) 
		return eval ( code, {'doc': doc, 't': doc[edit.o_start]} )
	except Exception as e:
		print("pass_filter ex:", e, edit)
	return False # ?

snt_errants =  lambda doc, edits : [ from_edit(doc, edit) for edit in edits if edit.o_start >= 0 and pass_filter(doc, edit)] 

def init_redis():
	import redis,json
	r = redis.Redis(host='dev.werror.com',port=4328, decode_responses=True)
	r.delete('errant:filter')
	r.delete('errant:type')
	r.hmset('errant:filter', errant_filter)
	r.hmset('errant:type', errant_type)

if __name__ == '__main__':
	pass
	#init_redis()


# 2022.2.24 https://redislite.readthedocs.io/en/latest/topic/what_is_redislite.html
from redislite import Redis
r = Redis('./dic.redislite')

from word_idf import word_idf
r.zadd('word_idf', word_idf )

from word_awl import word_awl 
for w in word_awl : 
	r.sadd('word_awl', w)
	r.hset('word_level', w, 'awl') 

from word_gsl1 import word_gsl1 
for w in word_gsl1 : 
	r.sadd('word_gsl1', w)
	r.hset('word_level', w, 'gsl1') 

from word_gsl2 import word_gsl2 
for w in word_gsl2 : 
	r.sadd('word_gsl2', w)
	r.hset('word_level', w, 'gsl2') 

from ecdic import ecdic 
r.hmset('ecdic', ecdic)

from bnc_wordlist import bnc_wordlist 
r.zadd('bnc_wordlist', bnc_wordlist)

print ("finished")



# 2021-2-6
import zipfile, pathlib,json

essay = '''   English is a internationaly language which becomes importantly for modern world.
    In China, English is took to be a foreigh language which many student choosed to learn. They begin to studying English at a early age. They use at least one hour to learn English knowledges a day. Even kids in kindergarten have begun learning simple words. That's a good phenomenan, for English is essential nowadays.
    In addition to, some people think English is superior than Chinese. In me opinion, though English is for great significance, but English is after all a foreign language. it is hard for people to see eye to eye. English do help us read English original works, but Chinese helps us learn a true China. Only by characters Chinese literature can send off its brilliance. Learning a country's culture, especial its classic culture, the first thing is learn its language. Because of we are Chinese, why do we give up our mother tongue and learn our owne culture through a foreign language?'''

def readline(infile, sepa=None):
	with open(infile, 'r') as fp:
		while True:
			line = fp.readline()
			if not line: break
			yield line.strip().split(sepa) if sepa else line.strip()

def word_level(): 
	''' {"w":"awl", "w2": "gsl1", "e3":"gsl2"} '''
	from dic import word_awl, word_gsl1, word_gsl2
	dic = {}
	[dic.update({w:"gsl1"}) for w in word_gsl1.word_gsl1]
	[dic.update({w:"gsl2"}) for w in word_gsl2.word_gsl2]
	[dic.update({w:"awl"}) for w in word_awl.word_awl]
	return dic

def readzip(name): # file endswith '.zip'
	z = zipfile.ZipFile( pathlib.Path(__file__).parent / f"{name}.zip" , 'r') 
	lines = z.read(z.namelist()[0]) #bytes 
	return lines.decode().strip().split("\n") #["'tween", "'tween decks", 'a.d.', 'a.k.a.', 'a.m.', 'aback', 'abaft'

def dm_essay(infile = '230537'): # 230537, [ {}, {}]
	lines = readzip(infile)
	return [ json.loads(line.strip().replace(': null, "',': 0, "'))  for line in lines ]

loadset = lambda name="verblist": { w.strip() for w in readzip( name ) if w } # one word, one line 
loadss = lambda name="ecdic": { line.split('\t')[0].strip():line.split('\t')[1].strip()  for line in readzip(name) if line } # s -> s , ecdic 

if __name__ == '__main__':
	print (dm_essay())

def word_map(chunk, idx , attr):  # by academic . | 1 | jj_to_nn 
	toks = chunk.split()
	w = toks[idx]
	toks[idx] = word_attr.get(w, {}).get(attr, '*' + attr)	
	return ' '.join(toks)

def word_attr_add(word, attr, val):
	if word in word_attr : 
		word_attr[word].update({attr:val})
	else :
		word_attr[word] = {attr:val}

stop_dobj_v = lambda : {'have','be','do','get'}
stop_dobj_n = lambda : {'thing'}
#map(f, iterable) ==> [f(x) for x in iterable]

def normalize(dct):
	v_sum = sum(dct.values()) + 1
	return { k:100*v/v_sum for k,v in dct.items()}

def code_dict(filename): 
	print (filename.split(".")[0] + " = {")
	for line in readlines(filename):
		if '"' in line:
			continue
		k,v = line.strip("\n").split("\t")  # one two:2,three:3
		print('"' + k  + '":{"' + v.replace(':','":').replace(',',',"') +"},")
	print ("}")
		
def readlines(filename):
	with open(filename, "r") as fp:
		return fp.readlines()